/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedClient, ObservedClientModel } from './ObservedClient';
import { Observer } from './Observer';
import { PartialBy } from './common/utils';
import { CallEventReport } from '@observertc/report-schemas-js';
import { ClientSample } from '@observertc/sample-schemas-js';
import { ObservedOutboundAudioTrack } from './ObservedOutboundAudioTrack';
import { ObservedOutboundVideoTrack } from './ObservedOutboundVideoTrack';
import { CalculatedScore } from './common/CalculatedScore';
export type ObservedCallModel = {
    serviceId: string;
    roomId: string;
    callId: string;
};
export type ObservedCallEvents = {
    update: [];
    newclient: [ObservedClient];
    callevent: [Omit<CallEventReport, 'callId' | 'serviceId' | 'roomId'>];
    empty: [];
    close: [];
};
export declare interface ObservedCall {
    on<U extends keyof ObservedCallEvents>(event: U, listener: (...args: ObservedCallEvents[U]) => void): this;
    off<U extends keyof ObservedCallEvents>(event: U, listener: (...args: ObservedCallEvents[U]) => void): this;
    once<U extends keyof ObservedCallEvents>(event: U, listener: (...args: ObservedCallEvents[U]) => void): this;
    emit<U extends keyof ObservedCallEvents>(event: U, ...args: ObservedCallEvents[U]): boolean;
}
export declare class ObservedCall<AppData extends Record<string, unknown> = Record<string, unknown>> extends EventEmitter {
    private readonly _model;
    readonly observer: Observer;
    readonly appData: AppData;
    readonly created: number;
    readonly processor: import("./common/Middleware").Processor<ClientSample>;
    private readonly _clients;
    _scoreData?: {
        score: CalculatedScore;
        calculated: number;
    };
    readonly sfuStreamIdToOutboundAudioTrack: Map<string | number, ObservedOutboundAudioTrack>;
    readonly sfuStreamIdToOutboundVideoTrack: Map<string | number, ObservedOutboundVideoTrack>;
    private _closed;
    started?: number;
    ended?: number;
    observationUpdated: number;
    observationEnded?: number;
    readonly observationStarted: number;
    constructor(_model: ObservedCallModel, observer: Observer, appData: AppData);
    get serviceId(): string;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get roomId(): string;
    get callId(): string;
    get clients(): ReadonlyMap<string, ObservedClient>;
    get score(): CalculatedScore | undefined;
    get closed(): boolean;
    close(): void;
    addEventReport(params: PartialBy<Omit<CallEventReport, 'serviceId' | 'roomId' | 'callId'>, 'timestamp'>): void;
    createClient<ClientAppData extends Record<string, unknown> = Record<string, unknown>>(config: ObservedClientModel & {
        appData: ClientAppData;
        joinedTimestamp?: number;
        createClientJoinedReport?: boolean;
        createClientLeftReport?: boolean;
    }): ObservedClient<ClientAppData>;
}
//# sourceMappingURL=ObservedCall.d.ts.map