"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedCall = void 0;
const events_1 = require("events");
const ObservedClient_1 = require("./ObservedClient");
const callEventReports_1 = require("./common/callEventReports");
const utils_1 = require("./common/utils");
const Middleware_1 = require("./common/Middleware");
class ObservedCall extends events_1.EventEmitter {
    constructor(_model, observer, appData) {
        super();
        this._model = _model;
        this.observer = observer;
        this.appData = appData;
        this.created = Date.now();
        this.processor = (0, Middleware_1.createProcessor)();
        this._clients = new Map();
        this.sfuStreamIdToOutboundAudioTrack = new Map();
        this.sfuStreamIdToOutboundVideoTrack = new Map();
        this._closed = false;
        this.observationUpdated = Date.now();
        this.observationStarted = Date.now();
        this.setMaxListeners(Infinity);
    }
    get serviceId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.serviceId;
    }
    get reports() {
        return this.observer.reports;
    }
    get roomId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.roomId;
    }
    get callId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.callId;
    }
    get clients() {
        return this._clients;
    }
    get score() {
        var _a, _b, _c;
        const now = Date.now();
        if (now - 15000 < ((_b = (_a = this._scoreData) === null || _a === void 0 ? void 0 : _a.calculated) !== null && _b !== void 0 ? _b : 0)) {
            return (_c = this._scoreData) === null || _c === void 0 ? void 0 : _c.score;
        }
        else if (this.clients.size < 1) {
            return {
                remarks: [{
                        severity: 'none',
                        text: 'No clients',
                    }],
                score: 5.0,
                timestamp: now,
            };
        }
        const scores = [...this.clients.values()].map((client) => { var _a, _b; return (_b = (_a = client.score) === null || _a === void 0 ? void 0 : _a.score) !== null && _b !== void 0 ? _b : 5.0; });
        this._scoreData = {
            score: {
                remarks: [],
                score: (0, utils_1.getMedian)(scores),
                timestamp: now,
            },
            calculated: now,
        };
        return this._scoreData.score;
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        Array.from(this._clients.values()).forEach((client) => client.close());
        this.observationEnded = Date.now();
        this.emit('close');
    }
    addEventReport(params) {
        var _a;
        this.reports.addCallEventReport(Object.assign(Object.assign({}, params), { serviceId: this.serviceId, roomId: this.roomId, callId: this.callId, timestamp: (_a = params.timestamp) !== null && _a !== void 0 ? _a : Date.now() }));
    }
    createClient(config) {
        var _a;
        if (this._closed)
            throw new Error(`Call ${this.callId} is closed`);
        const { appData, createClientJoinedReport, createClientLeftReport, joinedTimestamp = Date.now() } = config, model = __rest(config, ["appData", "createClientJoinedReport", "createClientLeftReport", "joinedTimestamp"]);
        const result = new ObservedClient_1.ObservedClient(model, this, appData);
        const onUpdate = ({ sample }) => {
            this.observationUpdated = Date.now();
            if (result.joined) {
                if (this.started === undefined || result.joined < this.started) {
                    this.started = result.joined;
                }
            }
            if (result.left) {
                if (this.ended === undefined || result.left > this.ended) {
                    this.ended = result.left;
                }
            }
            this.emit('update');
            this.processor.process(sample);
        };
        result.once('close', () => {
            result.off('update', onUpdate);
            this._clients.delete(result.clientId);
            if (this._clients.size === 0) {
                this.emit('empty');
            }
        });
        result.on('update', onUpdate);
        this._clients.set(result.clientId, result);
        if (createClientJoinedReport) {
            this.reports.addCallEventReport((0, callEventReports_1.createClientJoinedEventReport)(this.serviceId, result.mediaUnitId, this.roomId, this.callId, result.clientId, (_a = result.joined) !== null && _a !== void 0 ? _a : joinedTimestamp, result.userId, result.marker));
        }
        result.once('close', () => {
            var _a;
            if (createClientLeftReport) {
                this.reports.addCallEventReport((0, callEventReports_1.createClientLeftEventReport)(this.serviceId, result.mediaUnitId, this.roomId, this.callId, result.clientId, (_a = result.left) !== null && _a !== void 0 ? _a : Date.now(), result.userId, result.marker));
            }
        });
        this.emit('newclient', result);
        return result;
    }
}
exports.ObservedCall = ObservedCall;
