"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedClient = void 0;
const events_1 = require("events");
const ObservedPeerConnection_1 = require("./ObservedPeerConnection");
const logger_1 = require("./common/logger");
const CallMetaReports_1 = require("./common/CallMetaReports");
// eslint-disable-next-line camelcase
const utils_1 = require("./common/utils");
const CallEventType_1 = require("./common/CallEventType");
const logger = (0, logger_1.createLogger)('ObservedClient');
class ObservedClient extends events_1.EventEmitter {
    constructor(_model, call, appData) {
        super();
        this._model = _model;
        this.call = call;
        this.appData = appData;
        this.created = Date.now();
        this.updated = Date.now();
        this._peerConnections = new Map();
        this._closed = false;
        this._acceptedSamples = 0;
        this.usingTURN = false;
        this.availableOutgoingBitrate = 0;
        this.availableIncomingBitrate = 0;
        this.totalInboundPacketsLost = 0;
        this.totalInboundPacketsReceived = 0;
        this.totalOutboundPacketsSent = 0;
        this.totalDataChannelBytesSent = 0;
        this.totalDataChannelBytesReceived = 0;
        this.totalSentBytes = 0;
        this.totalReceivedBytes = 0;
        this.totalReceivedAudioBytes = 0;
        this.totalReceivedVideoBytes = 0;
        this.totalSentAudioBytes = 0;
        this.totalSentVideoBytes = 0;
        this.deltaReceivedAudioBytes = 0;
        this.deltaReceivedVideoBytes = 0;
        this.deltaSentAudioBytes = 0;
        this.deltaSentVideoBytes = 0;
        this.deltaDataChannelBytesSent = 0;
        this.deltaDataChannelBytesReceived = 0;
        this.deltaInboundPacketsLost = 0;
        this.deltaInboundPacketsReceived = 0;
        this.deltaOutboundPacketsSent = 0;
        this.mediaConstraints = [];
        this.mediaDevices = [];
        this.mediaCodecs = [];
        this.userMediaErrors = [];
        this.issues = [];
        this.ωpendingPeerConnectionTimestamps = [];
        this.ωpendingMediaTrackTimestamps = [];
        this.setMaxListeners(Infinity);
    }
    getSfu() {
        var _a;
        const sfu = this.call.observer.observedSfus.get((_a = this.sfuId) !== null && _a !== void 0 ? _a : '');
        if (!sfu)
            return;
        return sfu;
    }
    get joined() {
        return this._joined;
    }
    set joined(value) {
        this._joined = value;
        if (this._joined) {
            this.emit('joined');
        }
    }
    get left() {
        return this._left;
    }
    set left(value) {
        this._left = value;
        if (this._left) {
            this.emit('left');
        }
    }
    get sfu() {
        if (!this.sfuId)
            return;
        return this.call.observer.observedSfus.get(this.sfuId);
    }
    get coordinates() {
        return this._model.coordinates;
    }
    set coordinates(value) {
        this._model.coordinates = value;
    }
    get clientId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.clientId;
    }
    get roomId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.call.roomId;
    }
    get serviceId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.call.serviceId;
    }
    get callId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this.call.callId;
    }
    get mediaUnitId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.mediaUnitId;
    }
    get reports() {
        return this.call.reports;
    }
    get userId() {
        return this._model.userId;
    }
    set userId(userId) {
        this._model.userId = userId;
    }
    get timeZoneOffsetInHours() {
        return this._timeZoneOffsetInHours;
    }
    get marker() {
        return this._model.marker;
    }
    get browser() {
        return this._model.browser;
    }
    get engine() {
        return this._model.engine;
    }
    get operationSystem() {
        return this._model.operationSystem;
    }
    get platform() {
        return this._model.platform;
    }
    get acceptedSamples() {
        return this._acceptedSamples;
    }
    get closed() {
        return this._closed;
    }
    get peerConnections() {
        return this._peerConnections;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        Array.from(this._peerConnections.values()).forEach((peerConnection) => peerConnection.close());
        this.emit('close');
    }
    addEventReport(params) {
        var _a;
        const { attachments } = params, fields = __rest(params, ["attachments"]);
        this.reports.addCallEventReport(Object.assign(Object.assign({}, fields), { serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, roomId: this.roomId, callId: this.callId, clientId: this.clientId, userId: this.userId, timestamp: (_a = params.timestamp) !== null && _a !== void 0 ? _a : Date.now(), marker: this.marker, attachments: attachments ? JSON.stringify(attachments) : undefined }));
    }
    addExtensionStatsReport(extensionType, payload) {
        this.reports.addClientExtensionReport({
            serviceId: this.serviceId,
            mediaUnitId: this.mediaUnitId,
            roomId: this.roomId,
            callId: this.callId,
            clientId: this.clientId,
            userId: this.userId,
            timestamp: Date.now(),
            extensionType,
            payload: JSON.stringify(payload),
            marker: this.marker,
        });
    }
    addIssue(issue) {
        var _a;
        try {
            this.reports.addCallEventReport({
                serviceId: this.serviceId,
                mediaUnitId: this.mediaUnitId,
                roomId: this.roomId,
                callId: this.callId,
                clientId: this.clientId,
                userId: this.userId,
                name: CallEventType_1.CallEventType.CLIENT_ISSUE,
                value: issue.severity,
                peerConnectionId: issue.peerConnectionId,
                mediaTrackId: issue.trackId,
                message: issue.description,
                timestamp: (_a = issue.timestamp) !== null && _a !== void 0 ? _a : Date.now(),
                attachments: issue.attachments ? JSON.stringify(issue.attachments) : undefined,
            });
        }
        catch (err) {
            logger.warn(`Error adding client issue: ${err === null || err === void 0 ? void 0 : err.message}`);
        }
        this._addAndEmitIssue(issue);
    }
    accept(sample) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25;
        if (this._closed)
            throw new Error(`Client ${this.clientId} is closed`);
        if (sample.clientId && sample.clientId !== 'NULL' && sample.clientId !== this.clientId) {
            throw new Error(`Sample client id (${sample.clientId}) does not match the client id of the observed client (${this.clientId})`);
        }
        const now = Date.now();
        ++this._acceptedSamples;
        if (0 < this.issues.length) {
            // we reset the issues every time we accept a new sample
            this.issues = [];
        }
        for (const peerConnection of this._peerConnections.values()) {
            if (peerConnection.closed)
                continue;
            peerConnection.resetMetrics();
        }
        if (this._model.userId) {
            if (sample.userId && sample.userId !== 'NULL' && sample.userId !== this._model.userId) {
                this._model.userId = sample.userId;
            }
        }
        else if (sample.userId && sample.userId !== 'NULL') {
            this._model.userId = sample.userId;
        }
        if (this._model.marker !== sample.marker) {
            this._model.marker = sample.marker;
            this._peerConnections.forEach((peerConnection) => (peerConnection.marker = sample.marker));
        }
        if (this._timeZoneOffsetInHours !== sample.timeZoneOffsetInHours) {
            this._timeZoneOffsetInHours = sample.timeZoneOffsetInHours;
        }
        if (sample.os && (((_a = this._model.operationSystem) === null || _a === void 0 ? void 0 : _a.name) !== sample.os.name ||
            ((_b = this._model.operationSystem) === null || _b === void 0 ? void 0 : _b.version) !== sample.os.version ||
            ((_c = this._model.operationSystem) === null || _c === void 0 ? void 0 : _c.versionName) !== sample.os.versionName)) {
            this._model.operationSystem = sample.os;
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.OPERATION_SYSTEM,
                payload: sample.os,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
        }
        if (sample.engine && (((_d = this._model.engine) === null || _d === void 0 ? void 0 : _d.name) !== sample.engine.name ||
            ((_e = this._model.engine) === null || _e === void 0 ? void 0 : _e.version) !== sample.engine.version)) {
            this._model.engine = sample.engine;
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.ENGINE,
                payload: sample.engine,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
        }
        if (sample.platform && (((_f = this._model.platform) === null || _f === void 0 ? void 0 : _f.model) !== sample.platform.model ||
            ((_g = this._model.platform) === null || _g === void 0 ? void 0 : _g.type) !== sample.platform.type ||
            ((_h = this._model.platform) === null || _h === void 0 ? void 0 : _h.vendor) !== sample.platform.vendor)) {
            this._model.platform = sample.platform;
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.PLATFORM,
                payload: sample.platform,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
        }
        if (sample.browser && (((_j = this._model.browser) === null || _j === void 0 ? void 0 : _j.name) !== sample.browser.name ||
            ((_k = this._model.browser) === null || _k === void 0 ? void 0 : _k.version) !== sample.browser.version)) {
            this._model.browser = sample.browser;
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.BROWSER,
                payload: sample.browser,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
        }
        for (const mediaConstraint of (_l = sample.mediaConstraints) !== null && _l !== void 0 ? _l : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.MEDIA_CONSTRAINT,
                payload: mediaConstraint,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
            this.mediaConstraints.push(mediaConstraint);
        }
        for (const localSDP of (_m = sample.localSDPs) !== null && _m !== void 0 ? _m : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.LOCAL_SDP,
                payload: localSDP,
            }, this.userId, undefined, sample.timestamp);
            this.reports.addCallMetaReport(callMetaReport);
        }
        for (const extensionStats of (_o = sample.extensionStats) !== null && _o !== void 0 ? _o : []) {
            this.reports.addClientExtensionReport({
                serviceId: this.serviceId,
                mediaUnitId: this.mediaUnitId,
                roomId: this.roomId,
                callId: this.callId,
                clientId: this.clientId,
                userId: this.userId,
                timestamp: now,
                payload: extensionStats.payload,
                extensionType: extensionStats.type,
            });
        }
        for (let _26 of (_p = sample.customCallEvents) !== null && _p !== void 0 ? _p : []) {
            const { timestamp } = _26, callEvent = __rest(_26, ["timestamp"]);
            switch (callEvent.name) {
                case CallEventType_1.CallEventType.CLIENT_JOINED: {
                    const lastJoined = this.joined;
                    this.joined = timestamp;
                    // if it is joined before and it is joined again
                    if (lastJoined && this.joined && lastJoined !== this.joined) {
                        this.emit('rejoined', { lastJoined });
                    }
                    // in case we have a left event before the joined event
                    if (this.left && this.joined && this.left < this.joined) {
                        this.left = undefined;
                    }
                    break;
                }
                case CallEventType_1.CallEventType.CLIENT_LEFT: {
                    this.left = timestamp;
                    break;
                }
                case CallEventType_1.CallEventType.PEER_CONNECTION_OPENED: {
                    callEvent.peerConnectionId && this.ωpendingPeerConnectionTimestamps.push({
                        type: 'opened',
                        peerConnectionId: callEvent.peerConnectionId,
                        timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : sample.timestamp,
                    });
                    break;
                }
                case CallEventType_1.CallEventType.PEER_CONNECTION_CLOSED: {
                    callEvent.peerConnectionId && this.ωpendingPeerConnectionTimestamps.push({
                        type: 'closed',
                        peerConnectionId: callEvent.peerConnectionId,
                        timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : sample.timestamp,
                    });
                    break;
                }
                case CallEventType_1.CallEventType.MEDIA_TRACK_ADDED: {
                    callEvent.peerConnectionId && callEvent.mediaTrackId && this.ωpendingMediaTrackTimestamps.push({
                        type: 'added',
                        peerConnectionId: callEvent.peerConnectionId,
                        mediaTrackId: callEvent.mediaTrackId,
                        timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : sample.timestamp,
                    });
                    break;
                }
                case CallEventType_1.CallEventType.MEDIA_TRACK_REMOVED: {
                    callEvent.peerConnectionId && callEvent.mediaTrackId && this.ωpendingMediaTrackTimestamps.push({
                        type: 'removed',
                        peerConnectionId: callEvent.peerConnectionId,
                        mediaTrackId: callEvent.mediaTrackId,
                        timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : sample.timestamp,
                    });
                    break;
                }
                case CallEventType_1.CallEventType.CLIENT_ISSUE: {
                    const severity = callEvent.value ? callEvent.value : 'minor';
                    try {
                        const issue = {
                            severity,
                            timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : Date.now(),
                            description: callEvent.message,
                            peerConnectionId: callEvent.peerConnectionId,
                            trackId: callEvent.mediaTrackId,
                            attachments: callEvent.attachments ? JSON.parse(callEvent.attachments) : undefined,
                        };
                        this._addAndEmitIssue(issue);
                    }
                    catch (err) {
                        logger.warn(`Error parsing client issue: ${err === null || err === void 0 ? void 0 : err.message}`);
                    }
                    break;
                }
            }
            this.reports.addCallEventReport(Object.assign({ serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, roomId: this.roomId, callId: this.callId, clientId: this.clientId, userId: this.userId, timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : now }, callEvent));
            this.call.emit('callevent', Object.assign({ mediaUnitId: this.mediaUnitId, clientId: this.clientId, userId: this.userId, timestamp: timestamp !== null && timestamp !== void 0 ? timestamp : now }, callEvent));
        }
        for (const userMediaError of (_q = sample.userMediaErrors) !== null && _q !== void 0 ? _q : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.USER_MEDIA_ERROR,
                payload: userMediaError,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
            this.userMediaErrors.push(userMediaError);
            this.emit('usermediaerror', userMediaError);
        }
        for (const certificate of (_r = sample.certificates) !== null && _r !== void 0 ? _r : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.CERTIFICATE,
                payload: certificate,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
        }
        for (const codec of (_s = sample.codecs) !== null && _s !== void 0 ? _s : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.CODEC,
                payload: codec,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
            if (codec.mimeType && !this.mediaCodecs.find((c) => c.mimeType === codec.mimeType)) {
                this.mediaCodecs.push(codec);
            }
        }
        for (const iceServer of (_t = sample.iceServers) !== null && _t !== void 0 ? _t : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.ICE_SERVER,
                payload: iceServer,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
        }
        for (const mediaDevice of (_u = sample.mediaDevices) !== null && _u !== void 0 ? _u : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.MEDIA_DEVICE,
                payload: mediaDevice,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
            if (mediaDevice.id && !this.mediaDevices.find((d) => d.id === mediaDevice.id)) {
                this.mediaDevices.push(mediaDevice);
            }
        }
        for (const mediaSource of (_v = sample.mediaSources) !== null && _v !== void 0 ? _v : []) {
            const callMetaReport = (0, CallMetaReports_1.createCallMetaReport)(this.serviceId, this.mediaUnitId, this.roomId, this.callId, this.clientId, {
                type: CallMetaReports_1.CallMetaType.MEDIA_SOURCE,
                payload: mediaSource,
            }, this.userId);
            this.reports.addCallMetaReport(callMetaReport);
        }
        for (const transport of (_w = sample.pcTransports) !== null && _w !== void 0 ? _w : []) {
            try {
                const peerConnection = (_x = this._peerConnections.get(transport.transportId)) !== null && _x !== void 0 ? _x : this._createPeerConnection(transport.peerConnectionId);
                !peerConnection.label && transport.label && (peerConnection.label = transport.label);
                peerConnection.update(transport, sample.timestamp);
            }
            catch (err) {
                logger.error(`Error creating peer connection: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        for (const track of (_y = sample.inboundAudioTracks) !== null && _y !== void 0 ? _y : []) {
            if (!track.peerConnectionId || !track.trackId) {
                logger.warn(`InboundAudioTrack without peerConnectionId or trackId: ${JSON.stringify(track)}`);
                continue;
            }
            try {
                const peerConnection = (_z = this._peerConnections.get(track.peerConnectionId)) !== null && _z !== void 0 ? _z : this._createPeerConnection(track.peerConnectionId);
                if (!peerConnection)
                    continue;
                const inboundAudioTrack = (_0 = peerConnection.inboundAudioTracks.get(track.trackId)) !== null && _0 !== void 0 ? _0 : peerConnection.createInboundAudioTrack({
                    trackId: track.trackId,
                    sfuStreamId: track.sfuStreamId,
                    sfuSinkId: track.sfuSinkId,
                });
                inboundAudioTrack.update(track, sample.timestamp);
                if (!inboundAudioTrack.remoteOutboundTrack) {
                    const remoteOutboundTrack = this.call.sfuStreamIdToOutboundAudioTrack.get((_1 = inboundAudioTrack.sfuStreamId) !== null && _1 !== void 0 ? _1 : '');
                    if (remoteOutboundTrack) {
                        inboundAudioTrack.remoteOutboundTrack = remoteOutboundTrack;
                        inboundAudioTrack.once('close', () => {
                            var _a;
                            remoteOutboundTrack === null || remoteOutboundTrack === void 0 ? void 0 : remoteOutboundTrack.remoteInboundTracks.delete((_a = inboundAudioTrack.trackId) !== null && _a !== void 0 ? _a : '');
                        });
                        remoteOutboundTrack.remoteInboundTracks.set((_2 = inboundAudioTrack.trackId) !== null && _2 !== void 0 ? _2 : '', inboundAudioTrack);
                    }
                }
            }
            catch (err) {
                logger.error(`Error creating inbound audio track: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        for (const track of (_3 = sample.inboundVideoTracks) !== null && _3 !== void 0 ? _3 : []) {
            if (!track.peerConnectionId || !track.trackId) {
                logger.warn(`InboundVideoTrack without peerConnectionId or trackId: ${JSON.stringify(track)}`);
                continue;
            }
            if ((0, utils_1.isValidUuid)(track.trackId) === false) {
                // mediasoup-probator trackId is not a valid UUID, no need to warn about it
                if (track.trackId === 'probator')
                    continue;
                logger.warn(`InboundVideoTrack with invalid trackId: ${track.trackId}`);
                continue;
            }
            try {
                const peerConnection = (_4 = this._peerConnections.get(track.peerConnectionId)) !== null && _4 !== void 0 ? _4 : this._createPeerConnection(track.peerConnectionId);
                if (!peerConnection)
                    continue;
                const inboundVideoTrack = (_5 = peerConnection.inboundVideoTracks.get(track.trackId)) !== null && _5 !== void 0 ? _5 : peerConnection.createInboundVideoTrack({
                    trackId: track.trackId,
                    sfuStreamId: track.sfuStreamId,
                    sfuSinkId: track.sfuSinkId,
                });
                inboundVideoTrack.update(track, sample.timestamp);
                if (!inboundVideoTrack.remoteOutboundTrack) {
                    const remoteOutboundTrack = this.call.sfuStreamIdToOutboundVideoTrack.get((_6 = inboundVideoTrack.sfuStreamId) !== null && _6 !== void 0 ? _6 : '');
                    if (remoteOutboundTrack) {
                        inboundVideoTrack.remoteOutboundTrack = remoteOutboundTrack;
                        inboundVideoTrack.once('close', () => {
                            var _a;
                            remoteOutboundTrack === null || remoteOutboundTrack === void 0 ? void 0 : remoteOutboundTrack.remoteInboundTracks.delete((_a = inboundVideoTrack.trackId) !== null && _a !== void 0 ? _a : '');
                        });
                        remoteOutboundTrack.remoteInboundTracks.set((_7 = inboundVideoTrack.trackId) !== null && _7 !== void 0 ? _7 : '', inboundVideoTrack);
                    }
                }
            }
            catch (err) {
                logger.error(`Error creating inbound video track: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        for (const track of (_8 = sample.outboundAudioTracks) !== null && _8 !== void 0 ? _8 : []) {
            if (!track.peerConnectionId || !track.trackId) {
                logger.warn(`OutboundAudioTrack without peerConnectionId or trackId: ${JSON.stringify(track)}`);
                continue;
            }
            try {
                const peerConnection = (_9 = this._peerConnections.get(track.peerConnectionId)) !== null && _9 !== void 0 ? _9 : this._createPeerConnection(track.peerConnectionId);
                if (!peerConnection)
                    continue;
                const outboundAudioTrack = (_10 = peerConnection.outboundAudioTracks.get(track.trackId)) !== null && _10 !== void 0 ? _10 : peerConnection.createOutboundAudioTrack({
                    trackId: track.trackId,
                    sfuStreamId: track.sfuStreamId,
                });
                outboundAudioTrack.update(track, sample.timestamp);
                if (outboundAudioTrack.sfuStreamId && !this.call.sfuStreamIdToOutboundAudioTrack.has(outboundAudioTrack.sfuStreamId)) {
                    this.call.sfuStreamIdToOutboundAudioTrack.set(outboundAudioTrack.sfuStreamId, outboundAudioTrack);
                }
            }
            catch (err) {
                logger.error(`Error creating outbound audio track: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        for (const track of (_11 = sample.outboundVideoTracks) !== null && _11 !== void 0 ? _11 : []) {
            if (!track.peerConnectionId || !track.trackId) {
                logger.warn(`OutboundVideoTrack without peerConnectionId or trackId: ${JSON.stringify(track)}`);
                continue;
            }
            try {
                const peerConnection = (_12 = this._peerConnections.get(track.peerConnectionId)) !== null && _12 !== void 0 ? _12 : this._createPeerConnection(track.peerConnectionId);
                if (!peerConnection)
                    continue;
                const outboundVideoTrack = (_13 = peerConnection.outboundVideoTracks.get(track.trackId)) !== null && _13 !== void 0 ? _13 : peerConnection.createOutboundVideoTrack({
                    trackId: track.trackId,
                    sfuStreamId: track.sfuStreamId,
                });
                outboundVideoTrack.update(track, sample.timestamp);
                if (outboundVideoTrack.sfuStreamId && !this.call.sfuStreamIdToOutboundVideoTrack.has(outboundVideoTrack.sfuStreamId)) {
                    this.call.sfuStreamIdToOutboundVideoTrack.set(outboundVideoTrack.sfuStreamId, outboundVideoTrack);
                }
            }
            catch (err) {
                logger.error(`Error creating outbound video track: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        for (const iceLocalCandidate of (_14 = sample.iceLocalCandidates) !== null && _14 !== void 0 ? _14 : []) {
            if (!iceLocalCandidate.peerConnectionId) {
                logger.warn(`Local ice candidate without peerConnectionId: ${JSON.stringify(iceLocalCandidate)}`);
                continue;
            }
            const peerConnection = this._peerConnections.get(iceLocalCandidate.peerConnectionId);
            if (!peerConnection) {
                logger.debug(`Peer connection ${iceLocalCandidate.peerConnectionId} not found for ice local candidate ${iceLocalCandidate.id}`);
                continue;
            }
            peerConnection.ICE.addLocalCandidate(iceLocalCandidate, sample.timestamp);
        }
        for (const iceRemoteCandidate of (_15 = sample.iceRemoteCandidates) !== null && _15 !== void 0 ? _15 : []) {
            if (!iceRemoteCandidate.peerConnectionId) {
                logger.warn(`Remote ice candidate without peerConnectionId: ${JSON.stringify(iceRemoteCandidate)}`);
                continue;
            }
            const peerConnection = this._peerConnections.get(iceRemoteCandidate.peerConnectionId);
            if (!peerConnection) {
                logger.debug(`Peer connection ${iceRemoteCandidate.peerConnectionId} not found for ice remote candidate ${iceRemoteCandidate.id}`);
                continue;
            }
            peerConnection.ICE.addRemoteCandidate(iceRemoteCandidate, sample.timestamp);
        }
        for (const candidatePair of (_16 = sample.iceCandidatePairs) !== null && _16 !== void 0 ? _16 : []) {
            const peerConnection = this._peerConnections.get(candidatePair.peerConnectionId);
            if (!peerConnection) {
                logger.debug(`Peer connection ${candidatePair.peerConnectionId} not found for ice candidate pair ${candidatePair.localCandidateId}, ${candidatePair.remoteCandidateId}`);
                continue;
            }
            peerConnection.ICE.update(candidatePair, sample.timestamp);
        }
        for (const dataChannel of (_17 = sample.dataChannels) !== null && _17 !== void 0 ? _17 : []) {
            if (!dataChannel.peerConnectionId || !dataChannel.dataChannelIdentifier) {
                logger.warn(`DataChannel without peerConnectionId or dataChannelIdentifier: ${JSON.stringify(dataChannel)}`);
                continue;
            }
            try {
                const peerConnection = (_18 = this._peerConnections.get(dataChannel.peerConnectionId)) !== null && _18 !== void 0 ? _18 : this._createPeerConnection(dataChannel.peerConnectionId);
                if (!peerConnection)
                    continue;
                const observedDataChannel = (_19 = peerConnection.dataChannels.get(dataChannel.dataChannelIdentifier)) !== null && _19 !== void 0 ? _19 : peerConnection.createDataChannel(dataChannel.dataChannelIdentifier);
                observedDataChannel.update(dataChannel, sample.timestamp);
            }
            catch (err) {
                logger.error(`Error creating data channel: ${err === null || err === void 0 ? void 0 : err.message}`);
            }
        }
        // try to set the timestamps of the peer connections
        if (0 < this.ωpendingPeerConnectionTimestamps.length) {
            const newPendingTimestamps = [];
            for (const pendingTimestamp of this.ωpendingPeerConnectionTimestamps) {
                const peerConnection = this._peerConnections.get(pendingTimestamp.peerConnectionId);
                if (!peerConnection) {
                    newPendingTimestamps.push(pendingTimestamp);
                    continue;
                }
                if (pendingTimestamp.type === 'opened')
                    peerConnection.opened = pendingTimestamp.timestamp;
                else if (pendingTimestamp.type === 'closed')
                    peerConnection.closedTimestamp = pendingTimestamp.timestamp;
            }
            this.ωpendingPeerConnectionTimestamps = newPendingTimestamps;
        }
        // try to set the timestamps of the media tracks
        if (0 < this.ωpendingMediaTrackTimestamps.length) {
            const newPendingTimestamps = [];
            for (const pendingTimestamp of this.ωpendingMediaTrackTimestamps) {
                const peerConnection = this._peerConnections.get(pendingTimestamp.peerConnectionId);
                const mediaTrack = (_22 = (_21 = (_20 = peerConnection === null || peerConnection === void 0 ? void 0 : peerConnection.inboundAudioTracks.get(pendingTimestamp.mediaTrackId)) !== null && _20 !== void 0 ? _20 : peerConnection === null || peerConnection === void 0 ? void 0 : peerConnection.inboundVideoTracks.get(pendingTimestamp.mediaTrackId)) !== null && _21 !== void 0 ? _21 : peerConnection === null || peerConnection === void 0 ? void 0 : peerConnection.outboundAudioTracks.get(pendingTimestamp.mediaTrackId)) !== null && _22 !== void 0 ? _22 : peerConnection === null || peerConnection === void 0 ? void 0 : peerConnection.outboundVideoTracks.get(pendingTimestamp.mediaTrackId);
                if (!mediaTrack) {
                    newPendingTimestamps.push(pendingTimestamp);
                    continue;
                }
                if (pendingTimestamp.type === 'added')
                    mediaTrack.added = pendingTimestamp.timestamp;
                else if (pendingTimestamp.type === 'removed')
                    mediaTrack.removed = pendingTimestamp.timestamp;
            }
            this.ωpendingMediaTrackTimestamps = newPendingTimestamps;
        }
        // close resources that are not visited
        for (const peerConnection of this._peerConnections.values()) {
            if (!peerConnection.visited) {
                peerConnection.close();
                continue;
            }
            for (const track of peerConnection.inboundAudioTracks.values()) {
                if (!track.visited) {
                    track.close();
                    continue;
                }
                track.visited = false;
            }
            for (const track of peerConnection.inboundVideoTracks.values()) {
                if (!track.visited) {
                    track.close();
                    continue;
                }
                track.visited = false;
            }
            for (const track of peerConnection.outboundAudioTracks.values()) {
                if (!track.visited) {
                    track.close();
                    continue;
                }
                track.visited = false;
            }
            for (const dataChannel of peerConnection.dataChannels.values()) {
                if (!dataChannel.visited) {
                    dataChannel.close();
                    continue;
                }
                dataChannel.visited = false;
            }
            peerConnection.visited = false;
        }
        // update metrics
        const wasUsingTURN = this.usingTURN;
        const elapsedTimeInMs = now - this.updated;
        this.usingTURN = false;
        this.availableIncomingBitrate = 0;
        this.availableOutgoingBitrate = 0;
        this.deltaInboundPacketsLost = 0;
        this.deltaInboundPacketsReceived = 0;
        this.deltaOutboundPacketsSent = 0;
        this.deltaReceivedAudioBytes = 0;
        this.deltaReceivedVideoBytes = 0;
        this.deltaSentAudioBytes = 0;
        this.deltaSentVideoBytes = 0;
        this.deltaDataChannelBytesReceived = 0;
        this.deltaDataChannelBytesSent = 0;
        this.outboundAudioBitrate = 0;
        this.outboundVideoBitrate = 0;
        this.inboundAudioBitrate = 0;
        this.inboundVideoBitrate = 0;
        let sumRttInMs = 0;
        let anyPeerConnectionUsingTurn = false;
        let minPcScore;
        let maxPcScore;
        let numberOfScoredPeerConnections = 0;
        for (const peerConnection of this._peerConnections.values()) {
            if (peerConnection.closed)
                continue;
            peerConnection.updateMetrics();
            this.deltaInboundPacketsLost += peerConnection.deltaInboundPacketsLost;
            this.deltaInboundPacketsReceived += peerConnection.deltaInboundPacketsReceived;
            this.deltaOutboundPacketsSent += peerConnection.deltaOutboundPacketsSent;
            this.deltaReceivedAudioBytes += peerConnection.deltaReceivedAudioBytes;
            this.deltaReceivedVideoBytes += peerConnection.deltaReceivedVideoBytes;
            this.deltaSentAudioBytes += peerConnection.deltaSentAudioBytes;
            this.deltaSentVideoBytes += peerConnection.deltaSentVideoBytes;
            this.deltaDataChannelBytesReceived += peerConnection.deltaDataChannelBytesReceived;
            this.deltaDataChannelBytesSent += peerConnection.deltaDataChannelBytesSent;
            this.availableIncomingBitrate += (_23 = peerConnection.availableIncomingBitrate) !== null && _23 !== void 0 ? _23 : 0;
            this.availableOutgoingBitrate += (_24 = peerConnection.availableOutgoingBitrate) !== null && _24 !== void 0 ? _24 : 0;
            sumRttInMs += (_25 = peerConnection.avgRttInMs) !== null && _25 !== void 0 ? _25 : 0;
            anyPeerConnectionUsingTurn || (anyPeerConnectionUsingTurn = peerConnection.usingTURN);
            if (peerConnection.score) {
                if (minPcScore === undefined || peerConnection.score.score < minPcScore)
                    minPcScore = peerConnection.score.score;
                if (maxPcScore === undefined || peerConnection.score.score > maxPcScore)
                    maxPcScore = peerConnection.score.score;
                ++numberOfScoredPeerConnections;
            }
        }
        this.score = {
            score: minPcScore !== null && minPcScore !== void 0 ? minPcScore : -1,
            remarks: [{
                    severity: 'none',
                    text: `Min and max score of all peer connections: ${minPcScore}, ${maxPcScore}, number of PeerConnections with scores: ${numberOfScoredPeerConnections}`,
                }],
            timestamp: sample.timestamp,
        };
        this.emit('score', this.score);
        this.usingTURN = anyPeerConnectionUsingTurn === true;
        if (wasUsingTURN !== this.usingTURN)
            this.emit('usingturn', this.usingTURN);
        this.totalSentBytes += this.deltaSentAudioBytes + this.deltaSentVideoBytes;
        this.totalReceivedBytes += this.deltaReceivedAudioBytes + this.deltaReceivedVideoBytes;
        this.totalReceivedAudioBytes += this.deltaReceivedAudioBytes;
        this.totalReceivedVideoBytes += this.deltaReceivedVideoBytes;
        this.totalSentAudioBytes += this.deltaSentAudioBytes;
        this.totalSentVideoBytes += this.deltaSentVideoBytes;
        this.totalOutboundPacketsSent += this.deltaOutboundPacketsSent;
        this.totalInboundPacketsReceived += this.deltaInboundPacketsReceived;
        this.totalInboundPacketsLost += this.deltaInboundPacketsLost;
        this.totalDataChannelBytesSent += this.deltaDataChannelBytesSent;
        this.totalDataChannelBytesReceived += this.deltaDataChannelBytesReceived;
        this.outboundAudioBitrate = (this.deltaSentAudioBytes * 8) / (Math.max(elapsedTimeInMs, 1) / 1000);
        this.outboundVideoBitrate = (this.deltaSentVideoBytes * 8) / (Math.max(elapsedTimeInMs, 1) / 1000);
        this.inboundAudioBitrate = (this.deltaReceivedAudioBytes * 8) / (Math.max(elapsedTimeInMs, 1) / 1000);
        this.inboundVideoBitrate = (this.deltaReceivedVideoBytes * 8) / (Math.max(elapsedTimeInMs, 1) / 1000);
        this.avgRttInMs = this._peerConnections.size ? sumRttInMs / this._peerConnections.size : undefined;
        // to make sure when sample is emitted it can be associated to this client
        sample.clientId = this.clientId;
        this.updated = now;
        this.lastStatsTimestamp = sample.timestamp;
        this.emit('update', {
            sample,
            elapsedTimeInMs,
        });
    }
    _addAndEmitIssue(issue) {
        var _a;
        this.issues.push(issue);
        if (issue.peerConnectionId) {
            const peerConnection = this._peerConnections.get(issue.peerConnectionId);
            if (peerConnection) {
                if (issue.trackId) {
                    const track = (_a = peerConnection.inboundAudioTracks.get(issue.trackId)) !== null && _a !== void 0 ? _a : peerConnection.inboundVideoTracks.get(issue.trackId);
                    if (track)
                        track.ωpendingIssuesForScores.push(issue);
                }
                else {
                    peerConnection.ωpendingIssuesForScores.push(issue);
                }
            }
        }
        this.emit('issue', issue);
    }
    _createPeerConnection(peerConnectionId, label) {
        const result = new ObservedPeerConnection_1.ObservedPeerConnection({
            peerConnectionId,
            label,
        }, this);
        const onNewSelectedCandidatePair = ({ localCandidate, remoteCandidate }) => {
            this.emit('selectedcandidatepair', {
                peerConnection: result,
                localCandidate,
                remoteCandidate,
            });
        };
        result.once('close', () => {
            result.ICE.off('new-selected-candidate-pair', onNewSelectedCandidatePair);
            this._peerConnections.delete(peerConnectionId);
        });
        result.ICE.on('new-selected-candidate-pair', onNewSelectedCandidatePair);
        this._peerConnections.set(peerConnectionId, result);
        this.emit('newpeerconnection', result);
        return result;
    }
}
exports.ObservedClient = ObservedClient;
