/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { DataChannel } from '@observertc/sample-schemas-js';
export type ObservedDataChannelState = 'connecting' | 'open' | 'closing' | 'closed';
export type ObservedDataChannelModel = {
    channelId: number;
};
export type ObservedDataChannelEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    close: [];
};
export type ObservedInboundTrackStats = {
    ssrc: number;
    bitrate: number;
    fractionLost: number;
    rttInMs?: number;
    lostPackets?: number;
    receivedPackets?: number;
    receivedFrames?: number;
    decodedFrames?: number;
    droppedFrames?: number;
    receivedSamples?: number;
    silentConcealedSamples?: number;
    fractionLoss?: number;
};
export declare interface ObservedDataChannel {
    on<U extends keyof ObservedDataChannelEvents>(event: U, listener: (...args: ObservedDataChannelEvents[U]) => void): this;
    off<U extends keyof ObservedDataChannelEvents>(event: U, listener: (...args: ObservedDataChannelEvents[U]) => void): this;
    once<U extends keyof ObservedDataChannelEvents>(event: U, listener: (...args: ObservedDataChannelEvents[U]) => void): this;
    emit<U extends keyof ObservedDataChannelEvents>(event: U, ...args: ObservedDataChannelEvents[U]): boolean;
    update(sample: DataChannel, timestamp: number): void;
}
export declare class ObservedDataChannel extends EventEmitter {
    private readonly _model;
    readonly peerConnection: ObservedPeerConnection;
    readonly created: number;
    visited: boolean;
    private _stats?;
    private _closed;
    private _updated;
    bitrate: number;
    marker?: string;
    totalReceivedMessages: number;
    totalSentMessages: number;
    totalBytesReceived: number;
    totalBytesSent: number;
    deltaReceivedMessages: number;
    deltaSentMessages: number;
    deltaBytesReceived: number;
    deltaBytesSent: number;
    constructor(_model: ObservedDataChannelModel, peerConnection: ObservedPeerConnection);
    get serviceId(): string;
    get roomId(): string;
    get callId(): string;
    get clientId(): string;
    get mediaUnitId(): string;
    get peerConnectionId(): string;
    get channelId(): number;
    get updated(): number;
    get state(): ObservedDataChannelState | undefined;
    get label(): string | undefined;
    get protocol(): string | undefined;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get stats(): DataChannel | undefined;
    get closed(): boolean;
    close(): void;
    resetMetrics(): void;
}
//# sourceMappingURL=ObservedDataChannel.d.ts.map