"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedDataChannel = void 0;
const events_1 = require("events");
class ObservedDataChannel extends events_1.EventEmitter {
    constructor(_model, peerConnection) {
        super();
        this._model = _model;
        this.peerConnection = peerConnection;
        this.created = Date.now();
        this.visited = false;
        this._closed = false;
        this._updated = Date.now();
        this.bitrate = 0;
        this.totalReceivedMessages = 0;
        this.totalSentMessages = 0;
        this.totalBytesReceived = 0;
        this.totalBytesSent = 0;
        this.deltaReceivedMessages = 0;
        this.deltaSentMessages = 0;
        this.deltaBytesReceived = 0;
        this.deltaBytesSent = 0;
        this.setMaxListeners(Infinity);
    }
    get serviceId() {
        return this.peerConnection.serviceId;
    }
    get roomId() {
        return this.peerConnection.roomId;
    }
    get callId() {
        return this.peerConnection.callId;
    }
    get clientId() {
        return this.peerConnection.clientId;
    }
    get mediaUnitId() {
        return this.peerConnection.mediaUnitId;
    }
    get peerConnectionId() {
        return this.peerConnection.peerConnectionId;
    }
    get channelId() {
        return this._model.channelId;
    }
    get updated() {
        return this._updated;
    }
    get state() {
        var _a;
        return (_a = this._stats) === null || _a === void 0 ? void 0 : _a.state;
    }
    get label() {
        var _a;
        return (_a = this._stats) === null || _a === void 0 ? void 0 : _a.label;
    }
    get protocol() {
        var _a;
        return (_a = this._stats) === null || _a === void 0 ? void 0 : _a.protocol;
    }
    get reports() {
        return this.peerConnection.reports;
    }
    get stats() {
        return this._stats;
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        this.emit('close');
    }
    resetMetrics() {
        this.bitrate = 0;
        this.totalReceivedMessages = 0;
        this.totalSentMessages = 0;
        this.totalBytesReceived = 0;
        this.totalBytesSent = 0;
        this.deltaReceivedMessages = 0;
        this.deltaSentMessages = 0;
        this.deltaBytesReceived = 0;
    }
    update(sample, timestamp) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u;
        if (this._closed)
            return;
        const now = Date.now();
        const report = Object.assign(Object.assign({ serviceId: this.peerConnection.client.call.serviceId, roomId: this.peerConnection.client.call.roomId, callId: this.peerConnection.client.call.callId, clientId: this.peerConnection.client.clientId, userId: this.peerConnection.client.userId, mediaUnitId: this.peerConnection.client.mediaUnitId }, sample), { timestamp, sampleSeq: -1, marker: this.marker });
        this.reports.addClientDataChannelReport(report);
        sample.bytesReceived;
        sample.bytesSent;
        sample.messageReceived;
        sample.messageSent;
        const elapsedTimeInMs = Math.max(1, now - this._updated);
        this.bitrate = (((_a = sample.bytesReceived) !== null && _a !== void 0 ? _a : 0) - ((_c = (_b = this._stats) === null || _b === void 0 ? void 0 : _b.bytesReceived) !== null && _c !== void 0 ? _c : 0)) * 8 / (elapsedTimeInMs / 1000);
        this.deltaSentMessages = ((_d = sample.messageSent) !== null && _d !== void 0 ? _d : 0) - ((_f = (_e = this._stats) === null || _e === void 0 ? void 0 : _e.messageSent) !== null && _f !== void 0 ? _f : 0);
        this.deltaReceivedMessages = ((_g = sample.messageReceived) !== null && _g !== void 0 ? _g : 0) - ((_j = (_h = this._stats) === null || _h === void 0 ? void 0 : _h.messageReceived) !== null && _j !== void 0 ? _j : 0);
        this.deltaBytesSent = ((_k = sample.bytesSent) !== null && _k !== void 0 ? _k : 0) - ((_m = (_l = this._stats) === null || _l === void 0 ? void 0 : _l.bytesSent) !== null && _m !== void 0 ? _m : 0);
        this.deltaBytesReceived = ((_o = sample.bytesReceived) !== null && _o !== void 0 ? _o : 0) - ((_q = (_p = this._stats) === null || _p === void 0 ? void 0 : _p.bytesReceived) !== null && _q !== void 0 ? _q : 0);
        this.totalBytesReceived = (_r = sample.bytesReceived) !== null && _r !== void 0 ? _r : 0;
        this.totalBytesSent = (_s = sample.bytesSent) !== null && _s !== void 0 ? _s : 0;
        this.totalReceivedMessages = (_t = sample.messageReceived) !== null && _t !== void 0 ? _t : 0;
        this.totalSentMessages = (_u = sample.messageSent) !== null && _u !== void 0 ? _u : 0;
        this.visited = true;
        this._stats = sample;
        this._updated = timestamp;
        this.emit('update', {
            elapsedTimeInMs,
        });
    }
}
exports.ObservedDataChannel = ObservedDataChannel;
