/// <reference types="node" />
import { IceCandidatePair, IceLocalCandidate, IceRemoteCandidate } from '@observertc/sample-schemas-js';
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
export type ObservedICEEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    'new-local-candidate': [IceLocalCandidate];
    'new-remote-candidate': [IceRemoteCandidate];
    'new-selected-candidate-pair': [
        {
            localCandidate: IceLocalCandidate;
            remoteCandidate: IceRemoteCandidate;
        }
    ];
    usingturnchanged: [boolean];
    close: [];
};
export declare interface ObservedICE {
    on<U extends keyof ObservedICEEvents>(event: U, listener: (...args: ObservedICEEvents[U]) => void): this;
    off<U extends keyof ObservedICEEvents>(event: U, listener: (...args: ObservedICEEvents[U]) => void): this;
    once<U extends keyof ObservedICEEvents>(event: U, listener: (...args: ObservedICEEvents[U]) => void): this;
    emit<U extends keyof ObservedICEEvents>(event: U, ...args: ObservedICEEvents[U]): boolean;
}
export declare class ObservedICE extends EventEmitter {
    readonly peerConnection: ObservedPeerConnection;
    static create(peerConnection: ObservedPeerConnection): ObservedICE;
    marker?: string;
    private readonly _localCandidates;
    private readonly _remoteCandidates;
    usingTURN: boolean;
    deltaBytesReceived: number;
    deltaBytesSent: number;
    deltaPacketsReceived: number;
    deltaPacketsSent: number;
    totalBytesReceived: number;
    totalBytesSent: number;
    totalPacketsReceived: number;
    totalPacketsSent: number;
    currentRttInMs?: number;
    private _selectedLocalCandidateId?;
    private _selectedRemoteCandidateId?;
    private _updated;
    private _stats?;
    private _closed;
    private constructor();
    get reports(): import("./ReportsCollector").ReportsCollector;
    get localCandidates(): ReadonlyMap<string, IceLocalCandidate>;
    get remoteCandidates(): ReadonlyMap<string, IceRemoteCandidate>;
    get selectedLocalCandidate(): IceLocalCandidate | undefined;
    get selectedRemoteCandidate(): IceLocalCandidate | undefined;
    get stats(): IceCandidatePair | undefined;
    get updated(): number;
    get closed(): boolean;
    close(): void;
    addLocalCandidate(candidate: IceLocalCandidate, sampleTimestamp?: number): void;
    addRemoteCandidate(candidate: IceRemoteCandidate, sampleTimestamp?: number): void;
    resetMetrics(): void;
    update(candidatePair: IceCandidatePair, timestamp: number): void;
}
//# sourceMappingURL=ObservedICE.d.ts.map