/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { InboundAudioTrack } from '@observertc/sample-schemas-js';
import { CalculatedScore } from './common/CalculatedScore';
import { ClientIssue } from './monitors/CallSummary';
import { ObservedOutboundAudioTrack } from './ObservedOutboundAudioTrack';
export type ObservedInboundAudioTrackModel = {
    trackId: string;
    sfuStreamId?: string;
    sfuSinkId?: string;
};
export type ObservedInboundAudioTrackEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    score: [CalculatedScore];
    close: [];
    remoteoutboundtrack: [ObservedOutboundAudioTrack | undefined];
};
export type ObservedInboundAudioTrackStats = InboundAudioTrack & {
    ssrc: number;
    bitrate: number;
    fractionLost: number;
    rttInMs?: number;
    deltaReceivedBytes?: number;
    deltaLostPackets?: number;
    deltaReceivedPackets?: number;
    deltaReceivedFrames?: number;
    deltaDecodedFrames?: number;
    deltaDroppedFrames?: number;
    deltaReceivedSamples?: number;
    deltaSilentConcealedSamples?: number;
    fractionLoss?: number;
    statsTimestamp: number;
};
export declare interface ObservedInboundAudioTrack {
    on<U extends keyof ObservedInboundAudioTrackEvents>(event: U, listener: (...args: ObservedInboundAudioTrackEvents[U]) => void): this;
    off<U extends keyof ObservedInboundAudioTrackEvents>(event: U, listener: (...args: ObservedInboundAudioTrackEvents[U]) => void): this;
    once<U extends keyof ObservedInboundAudioTrackEvents>(event: U, listener: (...args: ObservedInboundAudioTrackEvents[U]) => void): this;
    emit<U extends keyof ObservedInboundAudioTrackEvents>(event: U, ...args: ObservedInboundAudioTrackEvents[U]): boolean;
    update(sample: InboundAudioTrack, timestamp: number): void;
}
export declare class ObservedInboundAudioTrack extends EventEmitter {
    private readonly _model;
    readonly peerConnection: ObservedPeerConnection;
    readonly created: number;
    visited: boolean;
    added?: number;
    removed?: number;
    private readonly _stats;
    private _closed;
    private _updated;
    private _remoteOutboundTrack?;
    private _lastMaxStatsTimestamp;
    bitrate: number;
    rttInMs?: number;
    jitter?: number;
    fractionLoss: number;
    marker?: string;
    totalLostPackets: number;
    totalReceivedPackets: number;
    totalBytesReceived: number;
    totalReceivedSamples: number;
    totalSilentConcealedSamples: number;
    deltaLostPackets: number;
    deltaReceivedPackets: number;
    deltaBytesReceived: number;
    deltaReceivedSamples: number;
    deltaSilentConcealedSamples: number;
    score?: CalculatedScore;
    ωpendingIssuesForScores: ClientIssue[];
    constructor(_model: ObservedInboundAudioTrackModel, peerConnection: ObservedPeerConnection);
    get serviceId(): string;
    get roomId(): string;
    get callId(): string;
    get clientId(): string;
    get mediaUnitId(): string;
    get peerConnectionId(): string;
    get trackId(): string;
    get sfuStreamId(): string | undefined;
    get sfuSinkId(): string | undefined;
    get updated(): number;
    get statsTimestamp(): number;
    get stats(): ReadonlyMap<number, ObservedInboundAudioTrackStats>;
    get reports(): import("./ReportsCollector").ReportsCollector;
    set remoteOutboundTrack(track: ObservedOutboundAudioTrack | undefined);
    get remoteOutboundTrack(): ObservedOutboundAudioTrack | undefined;
    get closed(): boolean;
    close(): void;
    updateMetrics(): void;
    private _updateQualityScore;
}
//# sourceMappingURL=ObservedInboundAudioTrack.d.ts.map