/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { InboundVideoTrack } from '@observertc/sample-schemas-js';
import { CalculatedScore } from './common/CalculatedScore';
import { ClientIssue } from './monitors/CallSummary';
import { ObservedOutboundVideoTrack } from './ObservedOutboundVideoTrack';
import { SupportedVideoCodecType } from './common/types';
export type ObservedInboundVideoTrackModel = {
    trackId: string;
    sfuStreamId?: string;
    sfuSinkId?: string;
};
export type ObservedInboundVideoTrackEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    score: [CalculatedScore];
    close: [];
    remoteoutboundtrack: [ObservedOutboundVideoTrack | undefined];
};
export type ObservedInboundVideoTrackStatsUpdate = InboundVideoTrack;
export type ObservedInboundVideoTrackStats = InboundVideoTrack & {
    ssrc: number;
    bitrate: number;
    fractionLost: number;
    rttInMs?: number;
    deltaReceivedBytes?: number;
    deltaLostPackets?: number;
    deltaReceivedPackets?: number;
    deltaReceivedFrames?: number;
    deltaDecodedFrames?: number;
    deltaDroppedFrames?: number;
    fractionLoss?: number;
    statsTimestamp: number;
};
export declare interface ObservedInboundVideoTrack {
    on<U extends keyof ObservedInboundVideoTrackEvents>(event: U, listener: (...args: ObservedInboundVideoTrackEvents[U]) => void): this;
    off<U extends keyof ObservedInboundVideoTrackEvents>(event: U, listener: (...args: ObservedInboundVideoTrackEvents[U]) => void): this;
    once<U extends keyof ObservedInboundVideoTrackEvents>(event: U, listener: (...args: ObservedInboundVideoTrackEvents[U]) => void): this;
    emit<U extends keyof ObservedInboundVideoTrackEvents>(event: U, ...args: ObservedInboundVideoTrackEvents[U]): boolean;
    update(sample: InboundVideoTrack, timestamp: number): void;
}
export declare class ObservedInboundVideoTrack extends EventEmitter {
    private readonly _model;
    readonly peerConnection: ObservedPeerConnection;
    readonly created: number;
    visited: boolean;
    added?: number;
    removed?: number;
    contentType: 'lowmotion' | 'standard' | 'highmotion';
    codec?: SupportedVideoCodecType;
    private readonly _stats;
    private _closed;
    private _updated;
    private _remoteOutboundTrack?;
    private _lastMaxStatsTimestamp;
    bitrate: number;
    rttInMs?: number;
    jitter?: number;
    fractionLoss: number;
    marker?: string;
    totalLostPackets: number;
    totalReceivedPackets: number;
    totalBytesReceived: number;
    totalReceivedFrames: number;
    totalDecodedFrames: number;
    totalDroppedFrames: number;
    deltaLostPackets: number;
    deltaReceivedPackets: number;
    deltaBytesReceived: number;
    deltaReceivedFrames: number;
    deltaDecodedFrames: number;
    deltaDroppedFrames: number;
    highestLayer?: ObservedInboundVideoTrackStats;
    score?: CalculatedScore;
    ωpendingIssuesForScores: ClientIssue[];
    constructor(_model: ObservedInboundVideoTrackModel, peerConnection: ObservedPeerConnection);
    get serviceId(): string;
    get roomId(): string;
    get callId(): string;
    get clientId(): string;
    get mediaUnitId(): string;
    get peerConnectionId(): string;
    get trackId(): string;
    get sfuStreamId(): string | undefined;
    get sfuSinkId(): string | undefined;
    get updated(): number;
    get statsTimestamp(): number;
    get stats(): ReadonlyMap<number, ObservedInboundVideoTrackStats>;
    get reports(): import("./ReportsCollector").ReportsCollector;
    set remoteOutboundTrack(track: ObservedOutboundVideoTrack | undefined);
    get remoteOutboundTrack(): ObservedOutboundVideoTrack | undefined;
    get closed(): boolean;
    close(): void;
    updateMetrics(): void;
    private _updateQualityScore;
}
//# sourceMappingURL=ObservedInboundVideoTrack.d.ts.map