/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { OutboundAudioTrack } from '@observertc/sample-schemas-js';
import { ObservedInboundAudioTrack } from './ObservedInboundAudioTrack';
import { CalculatedScore } from './common/CalculatedScore';
import { ClientIssue } from './monitors/CallSummary';
export type ObservedOutboundAudioTrackModel = {
    trackId: string;
    sfuStreamId?: string;
};
export type ObservedOutboundAudioTrackEvents = {
    qualitylimitationchanged: [string];
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    score: [CalculatedScore];
    close: [];
};
export type ObservedOutboundAudioTrackStats = OutboundAudioTrack & {
    ssrc: number;
    bitrate: number;
    rttInMs?: number;
    deltaLostPackets: number;
    deltaSentPackets: number;
    deltaSentBytes: number;
    deltaSentFrames?: number;
    deltaEncodedFrames?: number;
    statsTimestamp: number;
};
export declare interface ObservedOutboundAudioTrack {
    on<U extends keyof ObservedOutboundAudioTrackEvents>(event: U, listener: (...args: ObservedOutboundAudioTrackEvents[U]) => void): this;
    off<U extends keyof ObservedOutboundAudioTrackEvents>(event: U, listener: (...args: ObservedOutboundAudioTrackEvents[U]) => void): this;
    once<U extends keyof ObservedOutboundAudioTrackEvents>(event: U, listener: (...args: ObservedOutboundAudioTrackEvents[U]) => void): this;
    emit<U extends keyof ObservedOutboundAudioTrackEvents>(event: U, ...args: ObservedOutboundAudioTrackEvents[U]): boolean;
    update(sample: OutboundAudioTrack, timestamp: number): void;
}
export declare class ObservedOutboundAudioTrack extends EventEmitter {
    private readonly _model;
    readonly peerConnection: ObservedPeerConnection;
    readonly created: number;
    visited: boolean;
    added?: number;
    removed?: number;
    bitrate: number;
    rttInMs?: number;
    jitter?: number;
    marker?: string;
    totalLostPackets: number;
    totalSentPackets: number;
    totalSentBytes: number;
    totalSentFrames: number;
    deltaLostPackets: number;
    deltaSentPackets: number;
    deltaSentBytes: number;
    deltaSentFrames: number;
    deltaEncodedFrames: number;
    sendingBitrate: number;
    private readonly _stats;
    private _lastMaxStatsTimestamp;
    private _closed;
    private _updated;
    private _lastUpdateMetrics?;
    score?: CalculatedScore;
    ωpendingIssuesForScores: ClientIssue[];
    readonly remoteInboundTracks: Map<string, ObservedInboundAudioTrack>;
    constructor(_model: ObservedOutboundAudioTrackModel, peerConnection: ObservedPeerConnection);
    get serviceId(): string;
    get roomId(): string;
    get callId(): string;
    get clientId(): string;
    get mediaUnitId(): string;
    get peerConnectionId(): string;
    get trackId(): string;
    get sfuStreamId(): string | undefined;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get statsTimestamp(): number;
    get updated(): number;
    get stats(): ReadonlyMap<number, ObservedOutboundAudioTrackStats>;
    get closed(): boolean;
    close(): void;
    updateMetrics(): void;
    private _updateQualityScore;
}
//# sourceMappingURL=ObservedOutboundAudioTrack.d.ts.map