/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from './ObservedPeerConnection';
import { OutboundVideoTrack } from '@observertc/sample-schemas-js';
import { ObservedInboundVideoTrack } from './ObservedInboundVideoTrack';
import { CalculatedScore } from './common/CalculatedScore';
import { ClientIssue } from './monitors/CallSummary';
import { SupportedVideoCodecType } from './common/types';
export type ObservedOutboundVideoTrackModel = {
    trackId: string;
    sfuStreamId?: string;
};
export type ObservedOutboundVideoTrackEvents = {
    qualitylimitationchanged: [string];
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    score: [CalculatedScore];
    close: [];
};
export type ObservedOutboundVideoTrackStats = OutboundVideoTrack & {
    ssrc: number;
    bitrate: number;
    rttInMs?: number;
    deltaLostPackets: number;
    deltaSentPackets: number;
    deltaSentBytes: number;
    deltaSentFrames?: number;
    deltaEncodedFrames?: number;
    statsTimestamp: number;
};
export declare interface ObservedOutboundVideoTrack {
    on<U extends keyof ObservedOutboundVideoTrackEvents>(event: U, listener: (...args: ObservedOutboundVideoTrackEvents[U]) => void): this;
    off<U extends keyof ObservedOutboundVideoTrackEvents>(event: U, listener: (...args: ObservedOutboundVideoTrackEvents[U]) => void): this;
    once<U extends keyof ObservedOutboundVideoTrackEvents>(event: U, listener: (...args: ObservedOutboundVideoTrackEvents[U]) => void): this;
    emit<U extends keyof ObservedOutboundVideoTrackEvents>(event: U, ...args: ObservedOutboundVideoTrackEvents[U]): boolean;
    update(sample: OutboundVideoTrack, timestamp: number): void;
}
export declare class ObservedOutboundVideoTrack extends EventEmitter {
    private readonly _model;
    readonly peerConnection: ObservedPeerConnection;
    readonly created: number;
    visited: boolean;
    added?: number;
    removed?: number;
    contentType: 'lowmotion' | 'standard' | 'highmotion';
    codec?: SupportedVideoCodecType;
    highestLayer?: ObservedOutboundVideoTrackStats;
    bitrate: number;
    jitter?: number;
    rttInMs?: number;
    marker?: string;
    totalLostPackets: number;
    totalSentPackets: number;
    totalSentBytes: number;
    totalSentFrames: number;
    deltaLostPackets: number;
    deltaSentPackets: number;
    deltaSentBytes: number;
    deltaSentFrames: number;
    deltaEncodedFrames: number;
    sendingBitrate: number;
    private readonly _stats;
    private _lastMaxStatsTimestamp;
    private _closed;
    private _updated;
    readonly remoteInboundTracks: Map<string, ObservedInboundVideoTrack>;
    score?: CalculatedScore;
    ωpendingIssuesForScores: ClientIssue[];
    constructor(_model: ObservedOutboundVideoTrackModel, peerConnection: ObservedPeerConnection);
    get serviceId(): string;
    get roomId(): string;
    get callId(): string;
    get clientId(): string;
    get mediaUnitId(): string;
    get peerConnectionId(): string;
    get trackId(): string;
    get sfuStreamId(): string | undefined;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get updated(): number;
    get statsTimestamp(): number;
    get stats(): ReadonlyMap<number, ObservedOutboundVideoTrackStats>;
    get closed(): boolean;
    close(): void;
    private _lastUpdateMetrics?;
    updateMetrics(): void;
    private _updateQualityScore;
}
//# sourceMappingURL=ObservedOutboundVideoTrack.d.ts.map