"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedPeerConnection = void 0;
const events_1 = require("events");
const ObservedICE_1 = require("./ObservedICE");
const ObservedDataChannel_1 = require("./ObservedDataChannel");
const ObservedInboundAudioTrack_1 = require("./ObservedInboundAudioTrack");
const ObservedInboundVideoTrack_1 = require("./ObservedInboundVideoTrack");
const ObservedOutboundAudioTrack_1 = require("./ObservedOutboundAudioTrack");
const ObservedOutboundVideoTrack_1 = require("./ObservedOutboundVideoTrack");
const CalculatedScore_1 = require("./common/CalculatedScore");
class ObservedPeerConnection extends events_1.EventEmitter {
    constructor(_model, client) {
        super();
        this._model = _model;
        this.client = client;
        this.created = Date.now();
        this.visited = true;
        this._stabilityScores = [];
        this.ωpendingIssuesForScores = [];
        this.totalInboundPacketsLost = 0;
        this.totalInboundPacketsReceived = 0;
        this.totalOutboundPacketsSent = 0;
        this.totalDataChannelBytesSent = 0;
        this.totalDataChannelBytesReceived = 0;
        this.totalSentAudioBytes = 0;
        this.totalSentVideoBytes = 0;
        this.totalSentAudioPackets = 0;
        this.totalSentVideoPackets = 0;
        this.totalReceivedAudioPacktes = 0;
        this.totalReceivedVideoPackets = 0;
        this.totalReceivedAudioBytes = 0;
        this.totalReceivedVideoBytes = 0;
        this.deltaInboundPacketsLost = 0;
        this.deltaInboundPacketsReceived = 0;
        this.deltaOutboundPacketsSent = 0;
        this.deltaDataChannelBytesSent = 0;
        this.deltaDataChannelBytesReceived = 0;
        this.deltaInboundReceivedBytes = 0;
        this.deltaOutboundSentBytes = 0;
        this.deltaReceivedAudioBytes = 0;
        this.deltaReceivedVideoBytes = 0;
        this.deltaReceivedAudioPackets = 0;
        this.deltaReceivedVideoPackets = 0;
        this.deltaSentAudioBytes = 0;
        this.deltaSentVideoBytes = 0;
        this.receivingPacketsPerSecond = 0;
        this.sendingPacketsPerSecond = 0;
        this.sendingAudioBitrate = 0;
        this.sendingVideoBitrate = 0;
        this.receivingAudioBitrate = 0;
        this.receivingVideoBitrate = 0;
        this._closed = false;
        this._updated = Date.now();
        this.ICE = ObservedICE_1.ObservedICE.create(this);
        this._inboundAudioTracks = new Map();
        this._inboundVideoTracks = new Map();
        this._outboundAudioTracks = new Map();
        this._outboundVideoTracks = new Map();
        this._dataChannels = new Map();
        this.setMaxListeners(Infinity);
    }
    get serviceId() {
        return this.client.serviceId;
    }
    get roomId() {
        return this.client.roomId;
    }
    get callId() {
        return this.client.callId;
    }
    get clientId() {
        return this.client.clientId;
    }
    get mediaUnitId() {
        return this.client.mediaUnitId;
    }
    get label() {
        return this._model.label;
    }
    set label(value) {
        this._model.label = value;
    }
    get usingTURN() {
        return this.ICE.usingTURN;
    }
    get availableOutgoingBitrate() {
        var _a;
        return (_a = this.ICE.stats) === null || _a === void 0 ? void 0 : _a.availableOutgoingBitrate;
    }
    get availableIncomingBitrate() {
        var _a;
        return (_a = this.ICE.stats) === null || _a === void 0 ? void 0 : _a.availableIncomingBitrate;
    }
    get peerConnectionId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.peerConnectionId;
    }
    get reports() {
        return this.client.reports;
    }
    get stats() {
        return this._sample;
    }
    get updated() {
        return this._updated;
    }
    get inboundAudioTracks() {
        return this._inboundAudioTracks;
    }
    get inboundVideoTracks() {
        return this._inboundVideoTracks;
    }
    get outboundAudioTracks() {
        return this._outboundAudioTracks;
    }
    get outboundVideoTracks() {
        return this._outboundVideoTracks;
    }
    get dataChannels() {
        return this._dataChannels;
    }
    get uptimeInMs() {
        return this._updated - this.created;
    }
    get marker() {
        return this._marker;
    }
    set marker(value) {
        this._marker = value;
        this._inboundAudioTracks.forEach((track) => (track.marker = value));
        this._inboundVideoTracks.forEach((track) => (track.marker = value));
        this._outboundAudioTracks.forEach((track) => (track.marker = value));
        this._outboundVideoTracks.forEach((track) => (track.marker = value));
        this._dataChannels.forEach((channel) => (channel.marker = value));
        this.ICE.marker = value;
    }
    addEventReport(params) {
        var _a;
        this.reports.addCallEventReport(Object.assign(Object.assign({}, params), { serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, roomId: this.roomId, callId: this.callId, clientId: this.clientId, userId: this.client.userId, peerConnectionId: this.peerConnectionId, marker: this.client.marker, timestamp: (_a = params.timestamp) !== null && _a !== void 0 ? _a : Date.now() }));
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        Array.from(this._inboundAudioTracks.values()).forEach((track) => track.close());
        Array.from(this._inboundVideoTracks.values()).forEach((track) => track.close());
        Array.from(this._outboundAudioTracks.values()).forEach((track) => {
            var _a;
            this.client.call.sfuStreamIdToOutboundAudioTrack.delete((_a = track.sfuStreamId) !== null && _a !== void 0 ? _a : '');
            track.close();
        });
        Array.from(this._outboundVideoTracks.values()).forEach((track) => {
            var _a;
            this.client.call.sfuStreamIdToOutboundVideoTrack.delete((_a = track.sfuStreamId) !== null && _a !== void 0 ? _a : '');
            track.close();
        });
        this.emit('close');
    }
    getTrack(trackId) {
        var _a, _b, _c;
        return (_c = (_b = (_a = this._inboundAudioTracks.get(trackId)) !== null && _a !== void 0 ? _a : this._inboundVideoTracks.get(trackId)) !== null && _b !== void 0 ? _b : this._outboundAudioTracks.get(trackId)) !== null && _c !== void 0 ? _c : this._outboundVideoTracks.get(trackId);
    }
    resetMetrics() {
        this.deltaInboundPacketsLost = 0;
        this.deltaInboundPacketsReceived = 0;
        this.deltaOutboundPacketsSent = 0;
        this.deltaDataChannelBytesSent = 0;
        this.deltaDataChannelBytesReceived = 0;
        this.deltaInboundReceivedBytes = 0;
        this.deltaOutboundSentBytes = 0;
        this.deltaReceivedAudioBytes = 0;
        this.deltaReceivedVideoBytes = 0;
        this.deltaReceivedAudioPackets = 0;
        this.deltaReceivedVideoPackets = 0;
        this.deltaSentAudioBytes = 0;
        this.deltaSentVideoBytes = 0;
        this.sendingAudioBitrate = 0;
        this.sendingVideoBitrate = 0;
        this.receivingAudioBitrate = 0;
        this.receivingVideoBitrate = 0;
        this.ICE.resetMetrics();
    }
    update(sample, timestamp) {
        if (this._closed)
            return;
        if (sample.peerConnectionId !== this._model.peerConnectionId)
            throw new Error(`TransportId mismatch. PeerConnectionId: ${this._model.peerConnectionId} TransportId: ${sample.transportId}`);
        this._sample = sample;
        if (this._model.label !== sample.label) {
            this._model.label = sample.label;
        }
        const now = Date.now();
        const report = Object.assign(Object.assign({ serviceId: this.client.call.serviceId, roomId: this.client.call.roomId, callId: this.client.call.callId, clientId: this.client.clientId, userId: this.client.userId, mediaUnitId: this.client.mediaUnitId }, sample), { timestamp, sampleSeq: -1, marker: this.marker });
        this.reports.addPeerConnectionTransportReports(report);
        this._elapsedTimeSinceLastUpdate = now - this._updated;
        this.visited = true;
        this._updated = now;
        this._statsTimestamp = timestamp;
        this.emit('update', {
            elapsedTimeInMs: this._elapsedTimeSinceLastUpdate,
        });
    }
    createInboundAudioTrack(config) {
        if (this._closed)
            throw new Error(`PeerConnection ${this.peerConnectionId} is closed`);
        const result = new ObservedInboundAudioTrack_1.ObservedInboundAudioTrack(config, this);
        result.on('close', () => {
            this._inboundAudioTracks.delete(result.trackId);
        });
        this._inboundAudioTracks.set(result.trackId, result);
        this.emit('newinboudaudiotrack', result);
        return result;
    }
    createInboundVideoTrack(config) {
        if (this._closed)
            throw new Error(`PeerConnection ${this.peerConnectionId} is closed`);
        const result = new ObservedInboundVideoTrack_1.ObservedInboundVideoTrack(config, this);
        result.on('close', () => {
            this._inboundVideoTracks.delete(result.trackId);
        });
        this._inboundVideoTracks.set(result.trackId, result);
        this.emit('newinboudvideotrack', result);
        return result;
    }
    createOutboundAudioTrack(config) {
        if (this._closed)
            throw new Error(`PeerConnection ${this.peerConnectionId} is closed`);
        const result = new ObservedOutboundAudioTrack_1.ObservedOutboundAudioTrack(config, this);
        result.on('close', () => {
            this._outboundAudioTracks.delete(result.trackId);
        });
        this._outboundAudioTracks.set(result.trackId, result);
        this.emit('newoutboundaudiotrack', result);
        return result;
    }
    createOutboundVideoTrack(config) {
        if (this._closed)
            throw new Error(`PeerConnection ${this.peerConnectionId} is closed`);
        const result = new ObservedOutboundVideoTrack_1.ObservedOutboundVideoTrack(config, this);
        result.on('close', () => {
            this._outboundVideoTracks.delete(result.trackId);
        });
        this._outboundVideoTracks.set(result.trackId, result);
        this.emit('newoutboundvideotrack', result);
        return result;
    }
    createDataChannel(channelId) {
        if (this._closed)
            throw new Error(`PeerConnection ${this.peerConnectionId} is closed`);
        const result = new ObservedDataChannel_1.ObservedDataChannel({
            channelId,
        }, this);
        result.on('close', () => {
            this._dataChannels.delete(result.channelId);
        });
        this._dataChannels.set(result.channelId, result);
        this.emit('newdatachannel', result);
        return result;
    }
    updateMetrics() {
        var _a;
        let sumRttInMs = 0;
        let sumJitter = 0;
        const trackScores = [];
        this._inboundAudioTracks.forEach((track) => {
            var _a, _b;
            track.updateMetrics();
            this.deltaInboundPacketsLost += track.deltaLostPackets;
            this.deltaInboundPacketsReceived += track.deltaReceivedPackets;
            this.deltaInboundReceivedBytes += track.deltaBytesReceived;
            this.deltaReceivedAudioBytes += track.deltaBytesReceived;
            this.deltaReceivedAudioPackets += track.deltaReceivedPackets;
            this.receivingAudioBitrate += track.bitrate;
            sumRttInMs += ((_a = track.rttInMs) !== null && _a !== void 0 ? _a : 0);
            sumJitter += ((_b = track.jitter) !== null && _b !== void 0 ? _b : 0);
            track.score && trackScores.push(track.score);
        });
        this._inboundVideoTracks.forEach((track) => {
            var _a, _b;
            track.updateMetrics();
            this.deltaInboundPacketsLost += track.deltaLostPackets;
            this.deltaInboundPacketsReceived += track.deltaReceivedPackets;
            this.deltaInboundReceivedBytes += track.deltaBytesReceived;
            this.deltaReceivedVideoBytes += track.deltaBytesReceived;
            this.deltaReceivedVideoPackets += track.deltaReceivedPackets;
            this.receivingVideoBitrate += track.bitrate;
            sumRttInMs += ((_a = track.rttInMs) !== null && _a !== void 0 ? _a : 0);
            sumJitter += ((_b = track.jitter) !== null && _b !== void 0 ? _b : 0);
            track.score && trackScores.push(track.score);
        });
        this._outboundAudioTracks.forEach((track) => {
            var _a, _b;
            track.updateMetrics();
            this.deltaOutboundPacketsSent += track.deltaSentPackets;
            this.deltaOutboundSentBytes += track.deltaSentBytes;
            this.deltaSentAudioBytes += track.deltaSentBytes;
            this.deltaSentAudioBytes += track.deltaSentPackets;
            this.sendingAudioBitrate += track.bitrate;
            sumRttInMs += ((_a = track.rttInMs) !== null && _a !== void 0 ? _a : 0);
            sumJitter += ((_b = track.jitter) !== null && _b !== void 0 ? _b : 0);
            track.score && trackScores.push(track.score);
        });
        this._outboundVideoTracks.forEach((track) => {
            var _a, _b;
            track.updateMetrics();
            this.deltaOutboundPacketsSent += track.deltaSentPackets;
            this.deltaOutboundSentBytes += track.deltaSentBytes;
            this.deltaSentVideoBytes += track.deltaSentBytes;
            this.deltaSentVideoBytes += track.deltaSentPackets;
            this.sendingVideoBitrate += track.bitrate;
            sumRttInMs += ((_a = track.rttInMs) !== null && _a !== void 0 ? _a : 0);
            sumJitter += ((_b = track.jitter) !== null && _b !== void 0 ? _b : 0);
            track.score && trackScores.push(track.score);
        });
        this._dataChannels.forEach((channel) => {
            this.deltaDataChannelBytesSent += channel.deltaBytesSent;
            this.deltaDataChannelBytesReceived += channel.deltaBytesReceived;
        });
        const iceRttInMs = (_a = this.ICE.stats) === null || _a === void 0 ? void 0 : _a.currentRoundTripTime;
        let nrOfBelongings = this._inboundAudioTracks.size + this._inboundVideoTracks.size + this._outboundAudioTracks.size + this._outboundVideoTracks.size;
        this.avgJitter = 0 < nrOfBelongings ? sumJitter / nrOfBelongings : undefined;
        if (iceRttInMs) {
            sumRttInMs += iceRttInMs;
            nrOfBelongings += 1;
        }
        this.avgRttInMs = 0 < nrOfBelongings ? sumRttInMs / nrOfBelongings : undefined;
        this.totalDataChannelBytesReceived += this.deltaDataChannelBytesReceived;
        this.totalDataChannelBytesSent += this.deltaDataChannelBytesSent;
        this.totalInboundPacketsLost += this.deltaInboundPacketsLost;
        this.totalInboundPacketsReceived += this.deltaInboundPacketsReceived;
        this.totalOutboundPacketsSent += this.deltaOutboundPacketsSent;
        this.totalSentAudioBytes += this.deltaSentAudioBytes;
        this.totalSentVideoBytes += this.deltaSentVideoBytes;
        this.totalReceivedAudioBytes += this.deltaReceivedAudioBytes;
        this.totalReceivedVideoBytes += this.deltaReceivedVideoBytes;
        this.totalReceivedAudioPacktes += this.deltaReceivedAudioPackets;
        this.totalReceivedVideoPackets += this.deltaReceivedVideoPackets;
        this.totalSentAudioPackets += this.deltaSentAudioBytes;
        this.totalSentVideoPackets += this.deltaSentVideoBytes;
        if (this._elapsedTimeSinceLastUpdate && 0 < this._elapsedTimeSinceLastUpdate) {
            this.sendingPacketsPerSecond = this.deltaOutboundPacketsSent / (this._elapsedTimeSinceLastUpdate / 1000);
            this.receivingPacketsPerSecond = this.deltaInboundPacketsReceived / (this._elapsedTimeSinceLastUpdate / 1000);
        }
        else {
            this.sendingPacketsPerSecond = 0;
            this.receivingPacketsPerSecond = 0;
        }
        // calculate quality score
        this._updateQualityScore(trackScores);
    }
    _updateQualityScore(trackScores) {
        var _a, _b, _c, _d, _e, _f, _g;
        // Packet Jitter measured in seconds
        // we use RTT and lost packets to calculate the base score for the connection
        const rttInMs = (_a = this.avgRttInMs) !== null && _a !== void 0 ? _a : 0;
        const latencyFactor = rttInMs < 150 ? 1.0 : (0, CalculatedScore_1.getRttScore)(rttInMs);
        const totalPackets = Math.max(1, ((_b = this.totalInboundPacketsReceived) !== null && _b !== void 0 ? _b : 0) + ((_c = this.totalOutboundPacketsSent) !== null && _c !== void 0 ? _c : 0));
        const lostPackets = ((_d = this.totalInboundPacketsLost) !== null && _d !== void 0 ? _d : 0) + ((_e = this.deltaOutboundPacketsSent) !== null && _e !== void 0 ? _e : 0);
        const deliveryFactor = 1.0 - ((lostPackets) / (lostPackets + totalPackets));
        // let's push the actual stability score
        const stabilityScore = Math.pow(((latencyFactor * 0.5) + (deliveryFactor * 0.5)), 2);
        this._stabilityScores.push(stabilityScore);
        if (10 < this._stabilityScores.length) {
            this._stabilityScores.shift();
        }
        else if (this._stabilityScores.length < 5) {
            return;
        }
        let counter = 0;
        let weight = 0;
        let totalScore = 0;
        for (const score of this._stabilityScores) {
            weight += 1;
            counter += weight;
            totalScore += weight * score;
        }
        const weightedStabilityScore = totalScore / counter;
        let sumTrackScores = 0;
        for (const trackScore of trackScores) {
            sumTrackScores += trackScore.score;
        }
        const avgTrackScores = sumTrackScores / trackScores.length;
        const score = {
            remarks: [{
                    severity: 'none',
                    text: `Peer Connection stability score: ${weightedStabilityScore}`,
                },
                {
                    severity: 'none',
                    text: `Avg. Track score: ${avgTrackScores}`,
                }],
            score: Math.round(((weightedStabilityScore + avgTrackScores) * 5 * 100)) / 100.0,
            timestamp: (_f = this._statsTimestamp) !== null && _f !== void 0 ? _f : Date.now(),
        };
        for (const pendingIssue of this.ωpendingIssuesForScores) {
            switch (pendingIssue.severity) {
                case 'critical':
                    score.score = 0.0;
                    break;
                case 'major':
                    score.score *= 0.5;
                    break;
                case 'minor':
                    score.score *= 0.8;
                    break;
            }
            score.remarks.push({
                severity: pendingIssue.severity,
                text: (_g = pendingIssue.description) !== null && _g !== void 0 ? _g : 'Issue occurred',
            });
        }
        this.ωpendingIssuesForScores = [];
        this.score = score;
        this.emit('score', score);
    }
}
exports.ObservedPeerConnection = ObservedPeerConnection;
