/// <reference types="node" />
import { EventEmitter } from 'events';
import { Observer } from './Observer';
import { SfuSample } from '@observertc/sample-schemas-js';
import { ObservedSfuTransport } from './ObservedSfuTransport';
import { SfuEventReport } from '@observertc/report-schemas-js';
import { PartialBy } from './common/utils';
export type ObservedSfuModel = {
    serviceId: string;
    mediaUnitId: string;
    sfuId: string;
    joined?: number;
};
export type ObservedSfuEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    close: [];
    newtransport: [ObservedSfuTransport];
};
export declare interface ObservedSfu<AppData extends Record<string, unknown> = Record<string, unknown>> {
    on<U extends keyof ObservedSfuEvents>(event: U, listener: (...args: ObservedSfuEvents[U]) => void): this;
    off<U extends keyof ObservedSfuEvents>(event: U, listener: (...args: ObservedSfuEvents[U]) => void): this;
    once<U extends keyof ObservedSfuEvents>(event: U, listener: (...args: ObservedSfuEvents[U]) => void): this;
    emit<U extends keyof ObservedSfuEvents>(event: U, ...args: ObservedSfuEvents[U]): boolean;
    readonly appData: AppData;
    update(sample: SfuSample): void;
}
export declare class ObservedSfu<AppData extends Record<string, unknown> = Record<string, unknown>> extends EventEmitter {
    private readonly _model;
    readonly observer: Observer;
    readonly appData: AppData;
    readonly created: number;
    private _updated;
    private _closed;
    private _marker?;
    private _timeZoneOffsetInHours?;
    private _left?;
    private readonly _transports;
    constructor(_model: ObservedSfuModel, observer: Observer, appData: AppData);
    get marker(): string | undefined;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get serviceId(): string;
    get mediaUnitId(): string;
    get sfuId(): string;
    get joined(): number;
    get transports(): ReadonlyMap<string, ObservedSfuTransport>;
    get updated(): number;
    get closed(): boolean;
    close(): void;
    addEventReport(params: PartialBy<Omit<SfuEventReport, 'serviceId' | 'sfuId' | 'marker' | 'attachments'>, 'timestamp'> & {
        attachments?: Record<string, unknown>;
    }): void;
    addExtensionStatsReport(extensionType: string, payload?: Record<string, unknown>): void;
    private _createTransport;
}
//# sourceMappingURL=ObservedSfu.d.ts.map