"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedSfu = void 0;
const events_1 = require("events");
const ObservedSfuTransport_1 = require("./ObservedSfuTransport");
class ObservedSfu extends events_1.EventEmitter {
    constructor(_model, observer, appData) {
        super();
        this._model = _model;
        this.observer = observer;
        this.appData = appData;
        this.created = Date.now();
        this._updated = Date.now();
        this._closed = false;
        this._transports = new Map();
        this.setMaxListeners(Infinity);
    }
    get marker() {
        return this._marker;
    }
    get reports() {
        return this.observer.reports;
    }
    get serviceId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.serviceId;
    }
    get mediaUnitId() {
        return this._model.mediaUnitId;
    }
    get sfuId() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return this._model.sfuId;
    }
    get joined() {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return Number(this._model.joined);
    }
    get transports() {
        return this._transports;
    }
    get updated() {
        return this._updated;
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        [...this._transports.values()].forEach((transport) => transport.close());
        this.emit('close');
    }
    addEventReport(params) {
        var _a;
        const { attachments } = params, fields = __rest(params, ["attachments"]);
        this.reports.addCallEventReport(Object.assign(Object.assign({}, fields), { serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, marker: this.marker, timestamp: (_a = params.timestamp) !== null && _a !== void 0 ? _a : Date.now(), attachments: attachments ? JSON.stringify(attachments) : undefined }));
    }
    addExtensionStatsReport(extensionType, payload) {
        this.reports.addSfuExtensionReport({
            serviceId: this.serviceId,
            mediaUnitId: this.mediaUnitId,
            timestamp: Date.now(),
            extensionType,
            payload: payload ? JSON.stringify(payload) : undefined,
            marker: this.marker,
            sfuId: this.sfuId,
        });
    }
    update(sample) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
        if (this._closed)
            throw new Error(`Sfu ${this.sfuId} is closed`);
        if (sample.sfuId !== this.sfuId)
            throw new Error(`Sfu ${this.sfuId} is not the same as sample.sfuId`);
        if (!this._timeZoneOffsetInHours && sample.timeZoneOffsetInHours) {
            this._timeZoneOffsetInHours = sample.timeZoneOffsetInHours;
        }
        this._marker = sample.marker;
        const now = Date.now();
        const elapsedTimeInMs = now - this._updated;
        for (const customSfuEvents of (_a = sample.customSfuEvents) !== null && _a !== void 0 ? _a : []) {
            const report = Object.assign(Object.assign({ serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, sfuId: this.sfuId }, customSfuEvents), { timestamp: sample.timestamp, marker: this._marker });
            this.reports.addSfuEventReport(report);
        }
        for (const extensionStats of (_b = sample.extensionStats) !== null && _b !== void 0 ? _b : []) {
            const report = {
                serviceId: this.serviceId,
                mediaUnitId: this.mediaUnitId,
                sfuId: this.sfuId,
                extensionType: extensionStats.type,
                payload: extensionStats.payload,
                timestamp: sample.timestamp,
            };
            this.reports.addSfuExtensionReport(report);
        }
        for (const transport of (_c = sample.transports) !== null && _c !== void 0 ? _c : []) {
            const observedSfuTransport = (_d = this._transports.get(transport.transportId)) !== null && _d !== void 0 ? _d : this._createTransport({
                sfuTransportId: transport.transportId,
            });
            observedSfuTransport.update(transport, sample.timestamp);
        }
        for (const inboundRtpPad of (_e = sample.inboundRtpPads) !== null && _e !== void 0 ? _e : []) {
            const transport = (_f = this._transports.get(inboundRtpPad.transportId)) !== null && _f !== void 0 ? _f : this._createTransport({
                sfuTransportId: inboundRtpPad.transportId,
            });
            const observedSfuInboundRtpPad = (_g = transport.inboundRtpPads.get(inboundRtpPad.padId)) !== null && _g !== void 0 ? _g : transport.createSfuInboundRtpPad({
                sfuInboundRtpPadId: inboundRtpPad.padId,
            });
            observedSfuInboundRtpPad.update(inboundRtpPad, sample.timestamp);
        }
        for (const outboundRtpPad of (_h = sample.outboundRtpPads) !== null && _h !== void 0 ? _h : []) {
            const transport = (_j = this._transports.get(outboundRtpPad.transportId)) !== null && _j !== void 0 ? _j : this._createTransport({
                sfuTransportId: outboundRtpPad.transportId,
            });
            const observedSfuOutboundRtpPad = (_k = transport.outboundRtpPads.get(outboundRtpPad.padId)) !== null && _k !== void 0 ? _k : transport.createSfuOutboundRtpPad({
                sfuOutboundRtpPadId: outboundRtpPad.padId,
            });
            observedSfuOutboundRtpPad.update(outboundRtpPad, sample.timestamp);
        }
        for (const sctpChannel of (_l = sample.sctpChannels) !== null && _l !== void 0 ? _l : []) {
            const transport = (_m = this._transports.get(sctpChannel.transportId)) !== null && _m !== void 0 ? _m : this._createTransport({
                sfuTransportId: sctpChannel.transportId,
            });
            const observedSfuSctpChannel = (_o = transport.sctpChannels.get(sctpChannel.channelId)) !== null && _o !== void 0 ? _o : transport.createSfuSctpChannel({
                sfuSctpChannelId: sctpChannel.channelId,
            });
            observedSfuSctpChannel.update(sctpChannel, sample.timestamp);
        }
        this._updated = now;
        this.emit('update', {
            elapsedTimeInMs,
        });
    }
    _createTransport(model) {
        const result = new ObservedSfuTransport_1.ObservedSfuTransport(model, this);
        result.once('close', () => {
            this._transports.delete(model.sfuTransportId);
        });
        this._transports.set(model.sfuTransportId, result);
        this.emit('newtransport', result);
        return result;
    }
}
exports.ObservedSfu = ObservedSfu;
