"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedSfuOutboundRtpPad = void 0;
const events_1 = require("events");
class ObservedSfuOutboundRtpPad extends events_1.EventEmitter {
    constructor(_model, transport) {
        super();
        this._model = _model;
        this.transport = transport;
        this.created = Date.now();
        this._updated = Date.now();
        this._closed = false;
        this.setMaxListeners(Infinity);
    }
    get reports() {
        return this.transport.reports;
    }
    get serviceId() {
        return this.transport.serviceId;
    }
    get mediaUnitId() {
        return this.transport.mediaUnitId;
    }
    get sfuId() {
        return this.transport.sfuId;
    }
    get sfuTransportId() {
        return this.transport.sfuTransportId;
    }
    get sfuRtpPadId() {
        return this._model.sfuOutboundRtpPadId;
    }
    get updated() {
        return this._updated;
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        this.emit('close');
    }
    update(sample, timestamp) {
        const now = Date.now();
        const elapsedTimeInMs = now - this._updated;
        const { streamId: sfuStreamId, padId: rtpPadId, sinkId: sfuSinkId } = sample, reportData = __rest(sample, ["streamId", "padId", "sinkId"]);
        const report = Object.assign(Object.assign({ serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, sfuId: this.sfuId, callId: undefined, // TODO: where to get this from?
            sfuStreamId,
            sfuSinkId,
            rtpPadId }, reportData), { timestamp, marker: this.marker });
        this.reports.addSfuOutboundRtpPadReport(report);
        this.stats = sample;
        this._updated = now;
        this.emit('update', {
            elapsedTimeInMs,
        });
    }
}
exports.ObservedSfuOutboundRtpPad = ObservedSfuOutboundRtpPad;
