"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObservedSfuSctpChannel = void 0;
const events_1 = require("events");
class ObservedSfuSctpChannel extends events_1.EventEmitter {
    constructor(_model, transport) {
        super();
        this._model = _model;
        this.transport = transport;
        this.created = Date.now();
        this._updated = Date.now();
        this._closed = false;
        this.setMaxListeners(Infinity);
    }
    get serviceId() {
        return this.transport.serviceId;
    }
    get sfuId() {
        return this.transport.sfuId;
    }
    get mediaUnitId() {
        return this.transport.mediaUnitId;
    }
    get sfuTransportId() {
        return this.transport.sfuTransportId;
    }
    get sfuSctpChannelId() {
        return this._model.sfuSctpChannelId;
    }
    get reports() {
        return this.transport.reports;
    }
    get updated() {
        return this._updated;
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        this.emit('close');
    }
    update(sample, timestamp) {
        const now = Date.now();
        const elapsedTimeInMs = now - this._updated;
        const report = Object.assign(Object.assign({ serviceId: this.serviceId, mediaUnitId: this.mediaUnitId, sfuId: this.sfuId, callId: undefined, roomId: undefined }, sample), { timestamp, marker: this.marker });
        this.reports.addSfuTransportReport(report);
        this.stats = sample;
        this._updated = now;
        this.emit('update', {
            elapsedTimeInMs,
        });
    }
}
exports.ObservedSfuSctpChannel = ObservedSfuSctpChannel;
