/// <reference types="node" />
import { EventEmitter } from 'events';
import { SfuTransport } from '@observertc/sample-schemas-js';
import { ObservedSfu } from './ObservedSfu';
import { ObservedSfuInboundRtpPad, ObservedSfuInboundRtpPadModel } from './ObservedSfuInboundRtpPad';
import { ObservedSfuOutboundRtpPad, ObservedSfuOutboundRtpPadModel } from './ObservedSfuOutboundRtpPad';
import { ObservedSfuSctpChannel, ObservedSfuSctpChannelModel } from './ObservedSfuSctpChannel';
export type ObservedSfuTransportModel = {
    sfuTransportId: string;
};
export type ObservedSfuTransportEvents = {
    update: [
        {
            elapsedTimeInMs: number;
        }
    ];
    close: [];
    newsfuoutboundrtppad: [ObservedSfuOutboundRtpPad];
    newsfuinboundrtppad: [ObservedSfuInboundRtpPad];
    newsfusctpchannel: [ObservedSfuSctpChannel];
};
export declare interface ObservedSfuTransport {
    on<U extends keyof ObservedSfuTransportEvents>(event: U, listener: (...args: ObservedSfuTransportEvents[U]) => void): this;
    off<U extends keyof ObservedSfuTransportEvents>(event: U, listener: (...args: ObservedSfuTransportEvents[U]) => void): this;
    once<U extends keyof ObservedSfuTransportEvents>(event: U, listener: (...args: ObservedSfuTransportEvents[U]) => void): this;
    emit<U extends keyof ObservedSfuTransportEvents>(event: U, ...args: ObservedSfuTransportEvents[U]): boolean;
    update(sample: SfuTransport, timestamp: number): void;
}
export declare class ObservedSfuTransport extends EventEmitter {
    private readonly _model;
    readonly sfu: ObservedSfu;
    readonly created: number;
    stats?: SfuTransport;
    private _updated;
    private _closed;
    private _marker?;
    private readonly _inboundRtpPads;
    private readonly _outboundRtpPads;
    private readonly _sctpChannels;
    constructor(_model: ObservedSfuTransportModel, sfu: ObservedSfu);
    get inboundRtpPads(): ReadonlyMap<string, ObservedSfuInboundRtpPad>;
    get outboundRtpPads(): ReadonlyMap<string, ObservedSfuOutboundRtpPad>;
    get sctpChannels(): ReadonlyMap<string, ObservedSfuSctpChannel>;
    get serviceId(): string;
    get mediaUnitId(): string;
    get sfuId(): string;
    get sfuTransportId(): string;
    get reports(): import("./ReportsCollector").ReportsCollector;
    get updated(): number;
    get marker(): string | undefined;
    set marker(value: string | undefined);
    get closed(): boolean;
    close(): void;
    createSfuOutboundRtpPad(model: ObservedSfuOutboundRtpPadModel): ObservedSfuOutboundRtpPad;
    createSfuInboundRtpPad(model: ObservedSfuInboundRtpPadModel): ObservedSfuInboundRtpPad;
    createSfuSctpChannel(model: ObservedSfuSctpChannelModel): ObservedSfuSctpChannel;
}
//# sourceMappingURL=ObservedSfuTransport.d.ts.map