/// <reference types="node" />
import { ObservedCall, ObservedCallModel } from './ObservedCall';
import { ReportsCollector } from './ReportsCollector';
import { EventEmitter } from 'events';
import { PartialBy } from './common/utils';
import { ObserverSinkContext } from './common/types';
import { ObservedSfu, ObservedSfuModel } from './ObservedSfu';
import { CallSummaryMonitor, CallSummaryMonitorConfig } from './monitors/CallSummaryMonitor';
import { TurnUsageMonitor } from './monitors/TurnUsageMonitor';
import { ClientIssueMonitor } from './monitors/ClientIssueMonitor';
import { SfuServerMonitor } from './monitors/SfuServerMonitor';
export type ObserverEvents = {
    'newcall': [ObservedCall];
    'newsfu': [ObservedSfu];
    'reports': [ObserverSinkContext];
    'close': [];
};
export type ObserverConfig = {
    /**
     *
     * Sets the default serviceId for samples.
     *
     * For more information about a serviceId please visit https://observertc.org
     *
     */
    defaultServiceId: string;
    /**
     * Sets the default mediaUnitId for samples.
     *
     * For more information about a mediaUnitId please visit https://observertc.org
     */
    defaultMediaUnitId: string;
    maxReports?: number | undefined;
    maxCollectingTimeInMs?: number | undefined;
};
export declare interface Observer {
    on<U extends keyof ObserverEvents>(event: U, listener: (...args: ObserverEvents[U]) => void): this;
    off<U extends keyof ObserverEvents>(event: U, listener: (...args: ObserverEvents[U]) => void): this;
    once<U extends keyof ObserverEvents>(event: U, listener: (...args: ObserverEvents[U]) => void): this;
    emit<U extends keyof ObserverEvents>(event: U, ...args: ObserverEvents[U]): boolean;
}
export declare class Observer extends EventEmitter {
    readonly config: ObserverConfig;
    static create(providedConfig: Partial<ObserverConfig>): Observer;
    readonly reports: ReportsCollector;
    private readonly _observedCalls;
    private readonly _observedSfus;
    private readonly _monitors;
    private _reportTimer?;
    private _closed;
    constructor(config: ObserverConfig);
    createObservedCall<T extends Record<string, unknown> = Record<string, unknown>>(config: PartialBy<ObservedCallModel, 'serviceId'> & {
        appData: T;
        reportCallStarted?: boolean;
        reportCallEnded?: boolean;
    }): ObservedCall<T>;
    createObservedSfu<AppData extends Record<string, unknown> = Record<string, unknown>>(model: ObservedSfuModel, appData: AppData): ObservedSfu<AppData>;
    createCallSummaryMonitor(options?: CallSummaryMonitorConfig & {
        timeoutAfterCallClose?: number;
    }): CallSummaryMonitor;
    createTurnUsageMonitor(): TurnUsageMonitor;
    createSfuServerMonitor(): SfuServerMonitor;
    createClientIssueMonitor(): ClientIssueMonitor;
    get observedCalls(): ReadonlyMap<string, ObservedCall>;
    get observedSfus(): ReadonlyMap<string, ObservedSfu>;
    get closed(): boolean;
    close(): void;
    private _emitReports;
}
//# sourceMappingURL=Observer.d.ts.map