"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoteTrackAssigner = void 0;
class RemoteTrackAssigner {
    constructor() {
        this.sfuStreamIdToOutboundVideoTracks = new Map();
        this.sfuStreamIdToOutboundAudioTracks = new Map();
        this.pendingInboundVideoTracks = new Map();
        this.pendingInboundAudioTracks = new Map();
    }
    addOutboundAudioTrack(track) {
        const untilSfuStreamIdListener = () => {
            if (!track.sfuStreamId)
                return;
            this.sfuStreamIdToOutboundAudioTracks.set(track.sfuStreamId, track);
            track.off('update', untilSfuStreamIdListener);
            // we need to assign to the pending inbound tracks
        };
        track.on('update', untilSfuStreamIdListener);
        untilSfuStreamIdListener();
    }
    addOutboundVideoTrack(track) {
        const untilSfuStreamIdListener = () => {
            if (!track.sfuStreamId)
                return;
            this.sfuStreamIdToOutboundVideoTracks.set(track.sfuStreamId, track);
            track.off('update', untilSfuStreamIdListener);
            // we need to assign to the pending inbound tracks
        };
        track.on('update', untilSfuStreamIdListener);
        untilSfuStreamIdListener();
    }
    addInboundAudioTrack(track) {
        const untilSfuStreamIdListener = () => {
            if (!track.sfuStreamId)
                return;
            const outboundTrack = this.sfuStreamIdToOutboundAudioTracks.get(track.sfuStreamId);
            if (!outboundTrack) {
                this.pendingInboundAudioTracks.set(track.sfuStreamId, (this.pendingInboundAudioTracks.get(track.sfuStreamId) || []).concat(track));
                return;
            }
            if (outboundTrack) {
                track.remoteOutboundTrack = outboundTrack;
            }
            else {
                this.pendingInboundAudioTracks.set(track.sfuStreamId, (this.pendingInboundAudioTracks.get(track.sfuStreamId) || []).concat(track));
            }
        };
        track.on('update', untilSfuStreamIdListener);
    }
}
exports.RemoteTrackAssigner = RemoteTrackAssigner;
