import { CallEventReport, CallMetaReport, ClientDataChannelReport, ClientExtensionReport, IceCandidatePairReport, InboundAudioTrackReport, InboundVideoTrackReport, ObserverEventReport, OutboundAudioTrackReport, OutboundVideoTrackReport, PeerConnectionTransportReport, SfuEventReport, SfuExtensionReport, SfuInboundRtpPadReport, SfuMetaReport, SfuOutboundRtpPadReport, SfuSctpStreamReport, SFUTransportReport } from '@observertc/report-schemas-js';
import { ObserverSinkContext } from './common/types';
export type SinkEventsMap = {
    'call-event': {
        reports: CallEventReport[];
    };
    'call-meta': {
        reports: CallMetaReport[];
    };
    'client-data-channel': {
        reports: ClientDataChannelReport[];
    };
    'client-extension': {
        reports: ClientExtensionReport[];
    };
    'ice-candidate-pair': {
        reports: IceCandidatePairReport[];
    };
    'peer-connection-transport': {
        reports: PeerConnectionTransportReport[];
    };
    'inbound-audio-track': {
        reports: InboundAudioTrackReport[];
    };
    'inbound-video-track': {
        reports: InboundVideoTrackReport[];
    };
    'observer-event': {
        reports: ObserverEventReport[];
    };
    'outbound-audio-track': {
        reports: OutboundAudioTrackReport[];
    };
    'outbound-video-track': {
        reports: OutboundVideoTrackReport[];
    };
    'sfu-event': {
        reports: SfuEventReport[];
    };
    'sfu-extension': {
        reports: SfuExtensionReport[];
    };
    'sfu-inbound-rtp-pad': {
        reports: SfuInboundRtpPadReport[];
    };
    'sfu-outbound-rtp-pad': {
        reports: SfuOutboundRtpPadReport[];
    };
    'sfu-sctp-stream': {
        reports: SfuSctpStreamReport[];
    };
    'sfu-transport': {
        reports: SFUTransportReport[];
    };
    'sfu-meta': {
        reports: SfuMetaReport[];
    };
    'reports': ObserverSinkContext;
    'newreport': number;
};
export type ObserverSinkProcess = (observerSinkContext: ObserverSinkContext) => Promise<void>;
export declare class ReportsCollector {
    private _collectedReports;
    private _emitter;
    private _callEventReports;
    private _callMetaReports;
    private _clientDataChannelReports;
    private _clientExtensionReports;
    private _iceCandidatePairReports;
    private _inboundAudioTrackReports;
    private _inboundVideoTrackReports;
    private _peerConnectionTransportReports;
    private _observerEventReports;
    private _outboundAudioTrackReports;
    private _outboundVideoTrackReports;
    private _sfuEventReports;
    private _sfuExtensionReports;
    private _sfuInboundRtpPadReports;
    private _sfuOutboundRtpPadReports;
    private _sfuSctpStreamReports;
    private _sfuTransportReports;
    private _sfuMetaReports;
    constructor();
    on<K extends keyof SinkEventsMap>(event: K, listener: (reports: SinkEventsMap[K]) => void): this;
    off<K extends keyof SinkEventsMap>(event: K, listener: (reports: SinkEventsMap[K]) => void): this;
    once<K extends keyof SinkEventsMap>(event: K, listener: (reports: SinkEventsMap[K]) => void): this;
    addCallEventReport(report: CallEventReport): void;
    addCallMetaReport(report: CallMetaReport): void;
    addClientDataChannelReport(report: ClientDataChannelReport): void;
    addClientExtensionReport(report: ClientExtensionReport): void;
    addIceCandidatePairReport(report: IceCandidatePairReport): void;
    addInboundAudioTrackReport(report: InboundAudioTrackReport): void;
    addInboundVideoTrackReport(report: InboundVideoTrackReport): void;
    addPeerConnectionTransportReports(report: PeerConnectionTransportReport): void;
    addObserverEventReport(report: ObserverEventReport): void;
    addOutboundAudioTrackReport(report: OutboundAudioTrackReport): void;
    addOutboundVideoTrackReport(report: OutboundVideoTrackReport): void;
    addSfuEventReport(report: SfuEventReport): void;
    addSfuExtensionReport(report: SfuExtensionReport): void;
    addSfuInboundRtpPadReport(report: SfuInboundRtpPadReport): void;
    addSfuOutboundRtpPadReport(report: SfuOutboundRtpPadReport): void;
    addSfuSctpStreamReport(report: SfuSctpStreamReport): void;
    addSfuTransportReport(report: SFUTransportReport): void;
    addSfuMetaReport(report: SfuMetaReport): void;
    emit(): number;
    private _emit;
    private _createObserverSinkContext;
}
//# sourceMappingURL=ReportsCollector.d.ts.map