"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReportsCollector = void 0;
const events_1 = require("events");
class ReportsCollector {
    constructor() {
        this._collectedReports = 0;
        this._emitter = new events_1.EventEmitter();
        this._callEventReports = [];
        this._callMetaReports = [];
        this._clientDataChannelReports = [];
        this._clientExtensionReports = [];
        this._iceCandidatePairReports = [];
        this._inboundAudioTrackReports = [];
        this._inboundVideoTrackReports = [];
        this._peerConnectionTransportReports = [];
        this._observerEventReports = [];
        this._outboundAudioTrackReports = [];
        this._outboundVideoTrackReports = [];
        this._sfuEventReports = [];
        this._sfuExtensionReports = [];
        this._sfuInboundRtpPadReports = [];
        this._sfuOutboundRtpPadReports = [];
        this._sfuSctpStreamReports = [];
        this._sfuTransportReports = [];
        this._sfuMetaReports = [];
        // empty
    }
    on(event, listener) {
        this._emitter.addListener(event, listener);
        return this;
    }
    off(event, listener) {
        this._emitter.removeListener(event, listener);
        return this;
    }
    once(event, listener) {
        this._emitter.once(event, listener);
        return this;
    }
    addCallEventReport(report) {
        this._callEventReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addCallMetaReport(report) {
        this._callMetaReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addClientDataChannelReport(report) {
        this._clientDataChannelReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addClientExtensionReport(report) {
        this._clientExtensionReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addIceCandidatePairReport(report) {
        this._iceCandidatePairReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addInboundAudioTrackReport(report) {
        this._inboundAudioTrackReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addInboundVideoTrackReport(report) {
        this._inboundVideoTrackReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addPeerConnectionTransportReports(report) {
        this._peerConnectionTransportReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addObserverEventReport(report) {
        this._observerEventReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addOutboundAudioTrackReport(report) {
        this._outboundAudioTrackReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addOutboundVideoTrackReport(report) {
        this._outboundVideoTrackReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuEventReport(report) {
        this._sfuEventReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuExtensionReport(report) {
        this._sfuExtensionReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuInboundRtpPadReport(report) {
        this._sfuInboundRtpPadReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuOutboundRtpPadReport(report) {
        this._sfuOutboundRtpPadReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuSctpStreamReport(report) {
        this._sfuSctpStreamReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuTransportReport(report) {
        this._sfuTransportReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    addSfuMetaReport(report) {
        this._sfuMetaReports.push(report);
        this._emit('newreport', ++this._collectedReports);
    }
    emit() {
        let result = 0;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const checkAndEmit = (eventName, reports) => {
            if (0 < reports.length) {
                this._emit(eventName, { reports });
                result += reports.length;
            }
        };
        const context = this._createObserverSinkContext();
        checkAndEmit('call-event', this._callEventReports);
        this._callEventReports = [];
        checkAndEmit('call-meta', this._callMetaReports);
        this._callMetaReports = [];
        checkAndEmit('client-data-channel', this._clientDataChannelReports);
        this._clientDataChannelReports = [];
        checkAndEmit('client-extension', this._clientExtensionReports);
        this._clientExtensionReports = [];
        checkAndEmit('ice-candidate-pair', this._iceCandidatePairReports);
        this._iceCandidatePairReports = [];
        checkAndEmit('inbound-audio-track', this._inboundAudioTrackReports);
        this._inboundAudioTrackReports = [];
        checkAndEmit('inbound-video-track', this._inboundVideoTrackReports);
        this._inboundVideoTrackReports = [];
        checkAndEmit('peer-connection-transport', this._peerConnectionTransportReports);
        this._peerConnectionTransportReports = [];
        checkAndEmit('observer-event', this._observerEventReports);
        this._observerEventReports = [];
        checkAndEmit('outbound-audio-track', this._outboundAudioTrackReports);
        this._outboundAudioTrackReports = [];
        checkAndEmit('outbound-video-track', this._outboundVideoTrackReports);
        this._outboundVideoTrackReports = [];
        checkAndEmit('sfu-event', this._sfuEventReports);
        this._sfuEventReports = [];
        checkAndEmit('sfu-extension', this._sfuExtensionReports);
        this._sfuExtensionReports = [];
        checkAndEmit('sfu-inbound-rtp-pad', this._sfuInboundRtpPadReports);
        this._sfuInboundRtpPadReports = [];
        checkAndEmit('sfu-outbound-rtp-pad', this._sfuOutboundRtpPadReports);
        this._sfuOutboundRtpPadReports = [];
        checkAndEmit('sfu-sctp-stream', this._sfuSctpStreamReports);
        this._sfuSctpStreamReports = [];
        checkAndEmit('sfu-transport', this._sfuTransportReports);
        this._sfuTransportReports = [];
        checkAndEmit('sfu-meta', this._sfuMetaReports);
        this._sfuMetaReports = [];
        this._emit('reports', context);
        this._collectedReports = 0;
        return result;
    }
    _emit(event, reports) {
        return this._emitter.emit(event, reports);
    }
    _createObserverSinkContext() {
        const callEventReports = [...this._callEventReports];
        const callMetaReports = [...this._callMetaReports];
        const clientDataChannelReports = [...this._clientDataChannelReports];
        const clientExtensionReports = [...this._clientExtensionReports];
        const iceCandidatePairReports = [...this._iceCandidatePairReports];
        const inboundAudioTrackReports = [...this._inboundAudioTrackReports];
        const inboundVideoTrackReports = [...this._inboundVideoTrackReports];
        const peerConnectionTransportReports = [...this._peerConnectionTransportReports];
        const observerEventReports = [...this._observerEventReports];
        const outboundAudioTrackReports = [...this._outboundAudioTrackReports];
        const outboundVideoTrackReports = [...this._outboundVideoTrackReports];
        const sfuEventReports = [...this._sfuEventReports];
        const sfuExtensionReports = [...this._sfuExtensionReports];
        const sfuInboundRtpPadReports = [...this._sfuInboundRtpPadReports];
        const sfuOutboundRtpPadReports = [...this._sfuOutboundRtpPadReports];
        const sfuSctpStreamReports = [...this._sfuSctpStreamReports];
        const sfuTransportReports = [...this._sfuTransportReports];
        const sfuMetaReports = [...this._sfuMetaReports];
        return {
            callEventReports,
            callMetaReports,
            clientDataChannelReports,
            clientExtensionReports,
            iceCandidatePairReports,
            inboundAudioTrackReports,
            inboundVideoTrackReports,
            peerConnectionTransportReports,
            observerEventReports,
            outboundAudioTrackReports,
            outboundVideoTrackReports,
            sfuEventReports,
            sfuExtensionReports,
            sfuInboundRtpPadReports,
            sfuOutboundRtpPadReports,
            sfuSctpStreamReports,
            sfuTransportReports,
            sfuMetaReports,
        };
    }
}
exports.ReportsCollector = ReportsCollector;
