"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProcessor = void 0;
function createProcessor() {
    const stack = [];
    const result = {
        addMiddleware: (...middlewares) => {
            if (middlewares && 0 < middlewares.length) {
                stack.push(...middlewares);
            }
            return result;
        },
        removeMiddleware: (...middlewares) => {
            if (middlewares && 0 < middlewares.length) {
                for (const process of middlewares) {
                    const index = stack.indexOf(process);
                    if (0 < index)
                        stack.splice(index, 1);
                }
            }
            return result;
        },
        process: (value) => {
            let prevIndex = -1;
            const execute = (index, input) => {
                if (index <= prevIndex) {
                    throw new Error('middleware must call next() only once!');
                }
                prevIndex = index;
                const middleware = stack[index];
                if (!middleware)
                    return;
                const next = (nextInput) => execute(index + 1, nextInput);
                return middleware(input, next);
            };
            return execute(0, value);
        },
    };
    return result;
}
exports.createProcessor = createProcessor;
