"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLogger = exports.forwardLogsTo = exports.getLogLevel = exports.setLogLevel = void 0;
let logLevel = 'error';
const loggers = new Map();
let target = {
    // eslint-disable-next-line no-console
    trace: (...args) => console.log(...args),
    // eslint-disable-next-line no-console
    debug: (...args) => console.log(...args),
    // eslint-disable-next-line no-console
    info: (...args) => console.log(...args),
    // eslint-disable-next-line no-console
    warn: (...args) => console.warn(...args),
    // eslint-disable-next-line no-console
    error: (...args) => console.error(...args),
};
function setLogLevel(level) {
    logLevel = level;
    const moduleNames = Array.from(loggers.keys());
    loggers.clear();
    moduleNames.forEach((moduleName) => createLogger(moduleName));
}
exports.setLogLevel = setLogLevel;
const getLogLevel = () => {
    return logLevel;
};
exports.getLogLevel = getLogLevel;
function forwardLogsTo(logger) {
    target = logger;
}
exports.forwardLogsTo = forwardLogsTo;
const COLORS = {
    black: '\x1b[30m',
    red: '\x1b[31m',
    green: '\x1b[32m',
    yellow: '\x1b[33m',
    blue: '\x1b[34m',
    magenta: '\x1b[35m',
    cyan: '\x1b[36m',
    white: '\x1b[37m',
    default: '\x1b[39m',
};
function createLogger(moduleName) {
    if (loggers.has(moduleName)) {
        // eslint-disable-next-line no-console
        console.warn(`Logger for module ${moduleName} already exists`);
    }
    const trace = logLevel === 'trace' ? (message, ...args) => {
        target === null || target === void 0 ? void 0 : target.trace(`${COLORS.magenta}[TRACE]${COLORS.default} ${moduleName} ${message}`, ...args);
    } : () => void 0;
    const debug = logLevel === 'trace' || logLevel === 'debug' ? (message, ...args) => {
        target === null || target === void 0 ? void 0 : target.debug(`${COLORS.cyan}[DEBUG]${COLORS.default} ${moduleName} ${message}`, ...args);
    } : () => void 0;
    const info = logLevel === 'trace' || logLevel === 'debug' || logLevel === 'info' ? (message, ...args) => {
        target === null || target === void 0 ? void 0 : target.info(`${COLORS.green}[INFO]${COLORS.default} ${moduleName} ${message}`, ...args);
    } : () => void 0;
    const warn = logLevel === 'trace' || logLevel === 'debug' || logLevel === 'info' || logLevel === 'warn' ? (message, ...args) => {
        target === null || target === void 0 ? void 0 : target.warn(`${COLORS.yellow}[WARN]${COLORS.default} ${moduleName} ${message}`, ...args);
    } : () => void 0;
    const error = logLevel === 'trace' || logLevel === 'debug' || logLevel === 'info' || logLevel === 'warn' || logLevel === 'error' ? (message, ...args) => {
        target === null || target === void 0 ? void 0 : target.error(`${COLORS.red}[ERROR]${COLORS.default} ${moduleName} ${message}`, ...args);
    } : () => void 0;
    const logger = {
        trace,
        debug,
        info,
        warn,
        error,
    };
    loggers.set(moduleName, logger);
    return logger;
}
exports.createLogger = createLogger;
