/// <reference types="node" />
import { EventEmitter } from 'events';
import { CallSummary } from './CallSummary';
import { ObservedCall } from '../ObservedCall';
export type CallSummaryMonitorEvents = {
    close: [];
    summary: [CallSummary];
};
export type CallSummaryMonitorConfig = {
    detectUserMediaIssues?: boolean;
    detectMediaTrackQualityLimitationIssues?: boolean;
};
export declare interface CallSummaryMonitor {
    on<U extends keyof CallSummaryMonitorEvents>(event: U, listener: (...args: CallSummaryMonitorEvents[U]) => void): this;
    off<U extends keyof CallSummaryMonitorEvents>(event: U, listener: (...args: CallSummaryMonitorEvents[U]) => void): this;
    once<U extends keyof CallSummaryMonitorEvents>(event: U, listener: (...args: CallSummaryMonitorEvents[U]) => void): this;
    emit<U extends keyof CallSummaryMonitorEvents>(event: U, ...args: CallSummaryMonitorEvents[U]): boolean;
}
export declare class CallSummaryMonitor extends EventEmitter {
    readonly config: CallSummaryMonitorConfig;
    private readonly _summaries;
    private _closed;
    constructor(config?: CallSummaryMonitorConfig);
    addCall(call: ObservedCall): void;
    private _addClient;
    private _addPeerConnection;
    private _addOutboundVideoTrack;
    takeSummary(callId: string): CallSummary | undefined;
    get closed(): boolean;
    close(): void;
}
//# sourceMappingURL=CallSummaryMonitor.d.ts.map