"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientIssueMonitor = void 0;
const events_1 = require("events");
class ClientIssueMonitor extends events_1.EventEmitter {
    constructor(config = {}) {
        super();
        this.config = config;
        this._closed = false;
        this.setMaxListeners(Infinity);
    }
    addCall(call) {
        if (this.closed)
            return;
        const onNewClient = (client) => {
            this._addClient(call, client);
        };
        call.once('close', () => {
            call.off('newclient', onNewClient);
        });
        call.on('newclient', onNewClient);
    }
    _addClient(call, client) {
        if (this.closed)
            return;
        const { callId } = call;
        const { clientId } = client;
        const onIssue = (issue) => {
            const event = Object.assign(Object.assign({}, issue), { clientId,
                callId });
            this.emit(issue.severity, event);
            this.emit('issue', event);
        };
        client.once('close', () => {
            client.off('issue', onIssue);
        });
        client.on('issue', onIssue);
    }
    get closed() {
        return this._closed;
    }
    close() {
        if (this._closed)
            return;
        this._closed = true;
        this.emit('close');
    }
}
exports.ClientIssueMonitor = ClientIssueMonitor;
