/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from '../ObservedPeerConnection';
export type SfuServerMonitorMetricsRecord = {
    sfuId: string;
    receivedPacketsPerSecond: number;
    sentPacketsPerSecond: number;
    receivingAudioBitrate: number;
    receivingVideoBitrate: number;
    sendingAudioBitrate: number;
    sendingVideoBitrate: number;
    numberOfPeerConnectionsRttLt50: number;
    numberOfPeerConnectionsRttLt150: number;
    numberOfPeerConnectionsRttLt300: number;
    numberOfPeerConnectionsRttOver300: number;
    numberOfPeerConnections: number;
};
export type SfuServerMonitorConfig = {
    tooHighRttAlertSettings: {
        /**
         * The threshold for the RTT that will trigger the alert
         */
        threshold: 'rtt-gt-300' | 'rtt-gt-150' | 'rtt-gt-50';
        /**
         * The percentage of peer connections that need to be above the threshold to trigger the alert
         */
        percentageOfPeerConnectionsHighWatermark: number;
        /**
         * The percentage of peer connections that need to be above the threshold to clear the alert
         */
        percentageOfPeerConnectionsLowWatermark: number;
        /**
         * The minimum number of clients that need to be connected to trigger the alert
         */
        minNumberOfPeerConnections: number;
    };
};
export type SfuServerMonitorEvents = {
    'metrics': [SfuServerMonitorMetricsRecord];
    'high-rtt-started': [
        {
            sfuId: string;
            threshold: SfuServerMonitorConfig['tooHighRttAlertSettings']['threshold'];
            percentageOfPeerConnections: number;
        }
    ];
    'high-rtt-cleared': [
        {
            sfuId: string;
        }
    ];
    'too-many-clients': [{
        ip: string;
        numberOfClients: number;
    }];
    'close': [];
};
export declare interface SfuServerMonitor {
    on<U extends keyof SfuServerMonitorEvents>(event: U, listener: (...args: SfuServerMonitorEvents[U]) => void): this;
    off<U extends keyof SfuServerMonitorEvents>(event: U, listener: (...args: SfuServerMonitorEvents[U]) => void): this;
    once<U extends keyof SfuServerMonitorEvents>(event: U, listener: (...args: SfuServerMonitorEvents[U]) => void): this;
    emit<U extends keyof SfuServerMonitorEvents>(event: U, ...args: SfuServerMonitorEvents[U]): boolean;
}
export declare class SfuServerMonitor extends EventEmitter {
    readonly config: SfuServerMonitorConfig;
    private _closed;
    private readonly _peerConnections;
    private readonly _metrics;
    private readonly _rttAlertsOn;
    constructor(config: SfuServerMonitorConfig);
    get stats(): SfuServerMonitorMetricsRecord[];
    get closed(): boolean;
    close(): void;
    addPeerConnection(peerConnection: ObservedPeerConnection): void;
    private _checkHighRttForClients;
    private _increase;
    private _subtract;
}
//# sourceMappingURL=SfuServerMonitor.d.ts.map