/// <reference types="node" />
import { EventEmitter } from 'events';
import { ObservedPeerConnection } from '../ObservedPeerConnection';
import { ObservedClient } from '../ObservedClient';
export type TurnUsageMonitorEvents = {
    close: [];
    'newturnip': [string];
    update: [TurnUsage];
    addpeerconnection: [ObservedPeerConnection];
    removepeerconnection: [ObservedPeerConnection];
};
export type TurnUsage = {
    turnIp: string;
    totalBytesSent: number;
    totalBytesReceived: number;
    totalPacketsSent: number;
    totalPacketsReceived: number;
};
export type TurnStats = {
    turnIp: string;
    totalBytesSent: number;
    totalBytesReceived: number;
    totalPacketsSent: number;
    totalPacketsReceived: number;
    deltaBytesSent: number;
    deltaBytesReceived: number;
    timestamp: number;
    outboundBitrate: number;
    inboundBitrate: number;
};
export declare interface TurnUsageMonitor {
    on<U extends keyof TurnUsageMonitorEvents>(event: U, listener: (...args: TurnUsageMonitorEvents[U]) => void): this;
    off<U extends keyof TurnUsageMonitorEvents>(event: U, listener: (...args: TurnUsageMonitorEvents[U]) => void): this;
    once<U extends keyof TurnUsageMonitorEvents>(event: U, listener: (...args: TurnUsageMonitorEvents[U]) => void): this;
    emit<U extends keyof TurnUsageMonitorEvents>(event: U, ...args: TurnUsageMonitorEvents[U]): boolean;
}
export declare class TurnUsageMonitor extends EventEmitter {
    private readonly _connections;
    private readonly _turnUsage;
    private readonly _stats;
    private _closed;
    constructor();
    get stats(): TurnStats[];
    addPeerConnection(peerConnection: ObservedPeerConnection): void;
    removePeerConnection(peerConnection: ObservedPeerConnection): void;
    get turnIps(): string[];
    get peerConnections(): ObservedPeerConnection[];
    getUsage(turnIp: string): TurnUsage | undefined;
    get clients(): ReadonlyMap<string, ObservedClient>;
    get closed(): boolean;
    close(): void;
}
//# sourceMappingURL=TurnUsageMonitor.d.ts.map